from collections import OrderedDict as _OrderedDict
from collections import deque
from collections.abc import Callable, Iterable, Iterator, Mapping, MutableMapping
from typing import Any, TypeVar, overload

from typing_extensions import Self, override

__all__ = (
    "AttributeDict",
    "AttributeDictMixin",
    "BufferMap",
    "ChainMap",
    "ConfigurationView",
    "DictAttribute",
    "Evictable",
    "LimitedSet",
    "Messagebuffer",
    "OrderedDict",
    "force_mapping",
    "lpmerge",
)

_KT = TypeVar("_KT")
_VT = TypeVar("_VT")
_T = TypeVar("_T")

class OrderedDict(_OrderedDict[_KT, _VT]):
    def _LRUkey(self) -> Any: ...

def force_mapping(m: Mapping[_KT, _VT] | None) -> Mapping[_KT, _VT]: ...
def lpmerge(L: list[Any], R: list[Any]) -> list[Any]: ...

class AttributeDictMixin:
    def __getattr__(self, k: str) -> Any: ...
    @override
    def __setattr__(self, key: str, value: Any) -> None: ...

class AttributeDict(dict[str, Any], AttributeDictMixin):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class BufferMap(dict[_KT, _VT]):
    Buffer: type[Messagebuffer]
    Empty: type[BaseException]
    maxsize: int | None
    bufmaxsize: int | None
    total: int
    def __init__(
        self,
        maxsize: int,
        iterable: Iterable[tuple[_KT, _VT]] | None = None,
        bufmaxsize: int = 1000,
    ) -> None: ...
    def evict(self) -> None: ...
    def extend(self, key: _KT, it: Iterable[_VT]) -> None: ...
    def put(self, key: _KT, item: _VT) -> None: ...
    def take(self, key: _KT, *default: _VT) -> _VT: ...

class ChainMap(MutableMapping[str, Any]):
    maps: list[Mapping[str, Any]] | None
    changes: dict[str, Any] | None
    defaults: list[Mapping[str, Any]] | None
    key_t: type[str] | None
    def __init__(self, *maps: Mapping[str, Any], **kwargs: Any) -> None: ...
    def __bool__(self) -> bool: ...
    def __copy__(self) -> Self: ...
    @override
    def __getitem__(self, key: str) -> Any: ...
    def __missing__(self, key: str) -> Any: ...
    @override
    def __setitem__(self, key: str, value: Any) -> None: ...
    @override
    def __delitem__(self, key: str) -> None: ...
    @override
    def __iter__(self) -> Iterator[str]: ...
    @override
    def __len__(self) -> int: ...
    def add_defaults(self, d: Mapping[str, Any]) -> None: ...
    def bind_to(self, callback: Callable[[MutableMapping[str, Any]], None]) -> None: ...
    def copy(self) -> Self: ...
    @classmethod
    def fromkeys(cls, iterable: Iterable[str], *args: Any) -> Self: ...
    @overload
    def get(self, key: str) -> Any: ...
    @overload
    def get(self, key: str, default: _T) -> Any | _T: ...  # pyright: ignore[reportIncompatibleMethodOverride]
    def iteritems(self) -> Iterator[tuple[str, Any]]: ...
    def iterkeys(self) -> Iterator[str]: ...
    def itervalues(self) -> Iterator[Any]: ...
    @overload
    def pop(self, key: str) -> Any: ...
    @overload
    def pop(self, key: str, default: _T) -> Any | _T: ...  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def setdefault(self, key: str, default: Any = None) -> Any: ...

class ConfigurationView(ChainMap, AttributeDictMixin):
    def __init__(
        self,
        changes: dict[str, Any],
        defaults: Mapping[str, Any] | None = None,
        keys: Iterable[str] | None = None,
        prefix: str | None = None,
    ) -> None: ...
    def first(self, *keys: str) -> Any: ...
    @overload
    def get(self, key: str) -> Any: ...
    @overload
    def get(self, key: str, default: _T) -> Any | _T: ...
    def swap_with(self, other: ConfigurationView) -> None: ...

class DictAttribute(dict[str, Any]):
    obj: Any
    def __init__(self, obj: Any) -> None: ...
    @override
    def get(self, key: str, default: Any = None) -> Any: ...
    @override
    def setdefault(self, key: str, default: Any = None) -> Any: ...
    def iteritems(self) -> Iterator[tuple[str, Any]]: ...
    def iterkeys(self) -> Iterator[str]: ...
    def itervalues(self) -> Iterator[Any]: ...

class Evictable:
    Empty: type[BaseException]
    def evict(self) -> None: ...

class LimitedSet(Evictable):
    max_heap_percent_overload: int
    maxlen: int
    expires: float
    minlen: int
    def __init__(
        self,
        maxlen: int = 0,
        expires: float = 0,
        data: Iterable[Any] | None = None,
        minlen: int = 0,
    ) -> None: ...
    def add(self, item: Any, now: float | None = None) -> None: ...
    def as_dict(self) -> dict[Any, float]: ...
    def clear(self) -> None: ...
    def discard(self, item: Any) -> None: ...
    def pop(self, default: Any = None) -> Any: ...
    def pop_value(self, item: Any) -> None: ...
    def purge(self, now: float | None = None) -> None: ...
    def update(self, other: Iterable[Any]) -> None: ...
    def __contains__(self, value: object) -> bool: ...
    def __iter__(self) -> Iterator[Any]: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...

class Messagebuffer:
    Empty: type[BaseException]
    maxsize: int
    def __init__(
        self,
        maxsize: int,
        iterable: Iterable[Any] | None = None,
        deque: type[deque[Any]] = ...,
    ) -> None: ...
    def extend(self, it: Iterable[Any]) -> None: ...
    def put(self, item: Any) -> None: ...
    def take(self, *default: Any) -> Any: ...
    def __contains__(self, item: Any) -> bool: ...
    def __getitem__(self, index: int) -> Any: ...
    def __iter__(self) -> Iterator[Any]: ...
    def __len__(self) -> int: ...
    def __reversed__(self) -> Iterator[Any]: ...



from collections.abc import Callable
from types import TracebackType
from typing import Any

from typing_extensions import Self

__all__ = ("DummyLock", "LaxBoundedSemaphore")

class DummyLock:
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

class LaxBoundedSemaphore:
    initial_value: int
    value: int

    def __init__(self, value: int) -> None: ...
    def acquire(
        self, callback: Callable[..., None], *partial_args: Any, **partial_kwargs: Any
    ) -> bool: ...
    def release(self) -> None: ...
    def grow(self, n: int = ...) -> None: ...
    def shrink(self, n: int = ...) -> None: ...
    def clear(self) -> None: ...



from collections.abc import Callable, Iterable
from functools import partial
from re import Pattern
from typing import Any

__all__ = (
    "abbr",
    "abbrtask",
    "dedent",
    "dedent_initial",
    "ensure_newlines",
    "ensure_sep",
    "fill_paragraphs",
    "indent",
    "join",
    "pluralize",
    "pretty",
    "simple_format",
    "str_to_list",
    "truncate",
)

def abbr(S: str, max: int, ellipsis: str = "...") -> str: ...
def abbrtask(S: str, max: int) -> str: ...
def dedent(s: str, sep: str = "\n") -> str: ...
def dedent_initial(s: str, n: int = 4) -> str: ...
def ensure_sep(sep: str, s: str, n: int = 2) -> str: ...

ensure_newlines: partial[str]

def fill_paragraphs(s: str, width: int, sep: str = "\n") -> str: ...
def indent(t: str, indent: int = 0, sep: str = "\n") -> str: ...
def join(l: Iterable[str], sep: str = "\n") -> str: ...
def pluralize(n: int, text: str, suffix: str = "s") -> str: ...
def pretty(
    value: Any, width: int = 80, nl_width: int = 80, sep: str = "\n", **kw: Any
) -> str: ...
def simple_format(
    s: str,
    keys: dict[str, str | Callable[..., Any]],
    pattern: Pattern[str] = ...,
    expand: str = ...,
) -> str: ...
def str_to_list(s: str) -> list[str]: ...
def truncate(s: str, maxlen: int = 128, suffix: str = "...") -> str: ...



from typing import Any

from celery.loaders.base import BaseLoader
from typing_extensions import override

__all__ = ("DEFAULT_CONFIG_MODULE", "Loader")

DEFAULT_CONFIG_MODULE: str

class Loader(BaseLoader):
    @override
    def read_configuration(  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
        self,
        fail_silently: bool = True,  # type: ignore[override]
    ) -> dict[str, Any]: ...
    def setup_settings(self, settingsdict: dict[str, Any]) -> dict[str, Any]: ...



from typing import Any, TypeVar

from kombu.connection import Connection
from kombu.transport.base import StdChannel
from typing_extensions import Self, override

__all__ = ("MaybeChannelBound", "Object")

_T = TypeVar("_T")
_ObjectT = TypeVar("_ObjectT", bound=Object)

def _any(v: _T) -> _T: ...
def unpickle_dict(cls: type[_ObjectT], kwargs: dict[str, Any]) -> _ObjectT: ...

class Object:
    attrs: tuple[tuple[str, Any], ...]

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __copy__(self) -> Self: ...
    @override
    def __reduce__(self) -> tuple[Any, ...]: ...
    def as_dict(self, recurse: bool = ...) -> dict[str, Any]: ...

class MaybeChannelBound(Object):
    @property
    def can_cache_declaration(self) -> bool: ...
    def __call__(self, channel: StdChannel | Connection) -> Self: ...
    def bind(self, channel: StdChannel | Connection) -> Self: ...
    def maybe_bind(self, channel: StdChannel | Connection) -> Self: ...
    def revive(self, channel: StdChannel) -> None: ...
    def when_bound(self) -> None: ...
    def _repr_entity(self, item: str = ...) -> str: ...
    @property
    def channel(self) -> StdChannel: ...
    @property
    def is_bound(self) -> bool: ...



from collections.abc import Generator
from contextlib import contextmanager
from typing import Any

from celery import Celery

DEFAULT_TEST_CONFIG: dict[str, Any]

class Trap:
    def __getattr__(self, name: str) -> None: ...

class UnitLogging:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

def TestApp(
    name: str | None = None,
    config: dict[str, Any] | None = None,
    enable_logging: bool = False,
    set_as_current: bool = False,
    log: type[UnitLogging] | None = ...,
    backend: Any = None,
    broker: Any = None,
    **kwargs: Any,
) -> Celery: ...
@contextmanager
def set_trap(app: Celery) -> Generator[None, None, None]: ...
@contextmanager
def setup_default_app(
    app: Celery, use_trap: bool = False
) -> Generator[None, None, None]: ...



from collections.abc import Callable
from datetime import tzinfo
from types import ModuleType
from typing import Any

__all__ = ("Settings", "appstr", "bugreport", "filter_hidden_settings", "find_app")

from celery.app.base import Celery
from celery.utils.collections import ConfigurationView

def appstr(app: Celery) -> str: ...
def bugreport(app: Celery) -> str: ...
def filter_hidden_settings(conf: dict[str, Any]) -> dict[str, Any]: ...
def find_app(
    app: str,
    symbol_by_name: Callable[[str], Any] = ...,
    imp: Callable[[str], ModuleType] | None = ...,
) -> Celery: ...

class Settings(ConfigurationView):
    def __init__(
        self, *args: Any, deprecated_settings: set[str] | None = None, **kwargs: Any
    ) -> None: ...
    @property
    def broker_read_url(self) -> str: ...
    @property
    def broker_url(self) -> str: ...
    @property
    def broker_write_url(self) -> str: ...
    @property
    def result_backend(self) -> str: ...
    @property
    def task_default_exchange(self) -> str: ...
    @property
    def task_default_routing_key(self) -> str: ...
    @property
    def timezone(self) -> tzinfo: ...
    def finalize(self) -> None: ...
    def find_option(self, name: str, namespace: str = "") -> tuple[str, Any]: ...
    def find_value_for_key(self, name: str, namespace: str = "celery") -> Any: ...
    def get_by_parts(self, *parts: str) -> Any: ...
    def humanize(
        self, with_defaults: bool = False, censored: bool = True
    ) -> dict[str, Any]: ...
    def maybe_warn_deprecated_settings(self) -> None: ...
    def table(
        self, with_defaults: bool = False, censored: bool = True
    ) -> dict[str, Any]: ...
    def value_set_for(self, key: str) -> bool: ...
    def without_defaults(self) -> dict[str, Any]: ...



from typing import Any

__all__ = ("Message",)

class Message:
    CLASS_ID: int
    PROPERTIES: list[tuple[str, str]]

    body: bytes | str
    body_size: int
    body_received: int
    channel: Any
    delivery_info: dict[str, Any]
    properties: dict[str, Any]
    ready: bool
    frame_method: tuple[int, int] | None
    frame_args: bytes | None

    content_type: str | None
    content_encoding: str | None
    application_headers: dict[str, Any] | None
    delivery_mode: int | None
    priority: int | None
    correlation_id: str | None
    reply_to: str | None
    expiration: str | None
    message_id: str | None
    timestamp: int | None
    type: str | None
    user_id: str | None
    app_id: str | None
    cluster_id: str | None

    def __init__(
        self,
        body: bytes | str = ...,
        children: Any | None = ...,
        channel: Any | None = ...,
        **properties: Any,
    ) -> None: ...
    @property
    def headers(self) -> dict[str, Any] | None: ...
    @property
    def delivery_tag(self) -> int | None: ...



from typing import Any

__all__ = ()

def add_backend_cleanup_task(app: Any) -> Any: ...
def add_accumulate_task(app: Any) -> Any: ...
def add_chain_task(app: Any) -> Any: ...
def add_chord_task(app: Any) -> Any: ...
def add_chunk_task(app: Any) -> Any: ...
def add_group_task(app: Any) -> Any: ...
def add_map_task(app: Any) -> Any: ...
def add_starmap_task(app: Any) -> Any: ...
def add_unlock_chord_task(app: Any) -> Any: ...
def connect_on_app_finalize(callback: Any) -> Any: ...



from collections.abc import Iterable, Iterator
from typing import NamedTuple

class version_info_t(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: str
    serial: str

def version_string_as_tuple(s: str) -> version_info_t: ...
def fmatch_iter(
    needle: str, haystack: Iterable[str], min_ratio: float = ...
) -> Iterator[tuple[float, str]]: ...
def fmatch_best(
    needle: str, haystack: Iterable[str], min_ratio: float = ...
) -> str | None: ...
def escape_regex(p: str, white: str = ...) -> str: ...



import numbers
from collections.abc import Callable
from datetime import datetime, timedelta, tzinfo
from typing import Any, Literal, NamedTuple, TypeAlias

__all__ = (
    "ParseException",
    "crontab",
    "crontab_parser",
    "maybe_schedule",
    "schedule",
    "solar",
)

import ephem
from celery.app.base import Celery
from celery.utils.time import ffwd
from typing_extensions import override

class schedstate(NamedTuple):
    is_due: bool
    next: float

def cronfield(s: str | None) -> str: ...

class ParseException(Exception): ...

class BaseSchedule:
    nowfunc: Callable[[], datetime]
    def __init__(
        self,
        nowfun: Callable[[], datetime] | None = ...,
        app: Celery | None = ...,
    ) -> None: ...
    def now(self) -> datetime: ...
    def remaining_estimate(self, last_run_at: datetime) -> timedelta: ...
    def is_due(self, last_run_at: datetime) -> schedstate: ...
    def maybe_make_aware(self, dt: datetime, naive_as_utc: bool = True) -> datetime: ...
    @property
    def app(self) -> Celery: ...
    @app.setter
    def app(self, app: Celery) -> None: ...
    @property
    def tz(self) -> str: ...
    @property
    def utc_enabled(self) -> bool: ...
    def to_local(self, dt: datetime) -> datetime: ...
    @override
    def __eq__(self, other: object) -> bool: ...

class schedule(BaseSchedule):
    relative: bool
    def __init__(
        self,
        run_every: float | timedelta | None = ...,
        relative: bool = ...,
        nowfun: Callable[[], datetime] | None = ...,
        app: Celery | None = ...,
    ) -> None: ...
    @property
    def seconds(self) -> int: ...
    @property
    def human_seconds(self) -> str: ...

_ModuleLevelParseException = ParseException

class crontab_parser:
    ParseException: type[ParseException]
    def __init__(self, max_: int = ..., min_: int = ...) -> None: ...
    def parse(self, spec: str) -> set[int]: ...

Cronspec: TypeAlias = str | int | list[int]

class crontab(BaseSchedule):
    def __init__(
        self,
        minute: Cronspec = "*",
        hour: Cronspec = "*",
        day_of_week: Cronspec = "*",
        day_of_month: Cronspec = "*",
        month_of_year: Cronspec = "*",
        **kwargs: Any,
    ) -> None: ...
    @classmethod
    def from_string(cls, crontab: str) -> crontab: ...
    def remaining_delta(
        self, last_run_at: datetime, tz: tzinfo | None = None, ffwd: type[ffwd] = ...
    ) -> tuple[datetime, Any, datetime]: ...
    @override
    def remaining_estimate(
        self, last_run_at: datetime, ffwd: type[ffwd] = ...
    ) -> timedelta: ...

def maybe_schedule(
    s: numbers.Number | timedelta | BaseSchedule,
    relative: bool = False,
    app: Celery | None = None,
) -> float | timedelta | BaseSchedule: ...

_SolarEvent: TypeAlias = Literal[
    "dawn_astronomical",
    "dawn_nautical",
    "dawn_civil",
    "sunrise",
    "solar_noon",
    "sunset",
    "dusk_civil",
    "dusk_nautical",
    "dusk_astronomical",
]

class solar(BaseSchedule):
    cal: ephem.Observer
    event: _SolarEvent
    lat: float
    lon: float
    def __init__(
        self,
        event: _SolarEvent,
        lat: float,
        lon: float,
        **kwargs: Any,
    ) -> None: ...



from collections.abc import Callable
from typing import Any

from celery.app.base import Celery
from celery.app.task import Task
from celery.local import Proxy

__all__ = (
    "connect_on_app_finalize",
    "current_app",
    "current_task",
    "get_current_app",
    "get_current_task",
    "get_current_worker_task",
    "set_default_app",
)

current_app: Proxy[Celery]
current_task: Proxy[Task[Any, Any]]

def get_current_task() -> Task[Any, Any]: ...
def get_current_app() -> Celery: ...
def get_current_worker_task() -> Task[Any, Any] | None: ...
def set_default_app(app: Celery) -> None: ...
def connect_on_app_finalize(
    callback: Callable[[Celery], Any],
) -> Callable[[Celery], Any]: ...



from logging import Logger
from typing import Any

__all__ = ("Logwrapped", "setup_logging")

class Logwrapped:
    instance: Any
    logger: Logger
    ident: str | None

    def __init__(
        self, instance: Any, logger: Logger | None = ..., ident: str | None = ...
    ) -> None: ...
    def __getattr__(self, key: str) -> Any: ...

def setup_logging(
    loglevel: int | None = ..., loggers: list[str] | None = ...
) -> None: ...



from collections.abc import Callable, Iterable, Sequence
from types import ModuleType
from typing import Any, TypeVar

import kombu
from kombu.utils.functional import LRUCache
from kombu.utils.objects import cached_property as cached_property
from kombu.utils.uuid import uuid as uuid

__all__ = (
    "LOG_LEVELS",
    "cached_property",
    "chunks",
    "gen_task_name",
    "gen_unique_id",
    "get_cls_by_name",
    "get_full_cls_name",
    "import_from_cwd",
    "instantiate",
    "memoize",
    "nodename",
    "nodesplit",
    "noop",
    "uuid",
    "worker_direct",
)

LOG_LEVELS: dict[str, int]
nodenames: ModuleType
_T = TypeVar("_T")

def chunks(it: Iterable[_T], n: int) -> Iterable[Sequence[_T]]: ...
def gen_task_name(app: Any, name: str, module_name: str) -> str: ...
def gen_unique_id() -> str: ...
def get_cls_by_name(
    name: str,
    aliases: dict[str, str] | None = None,
    imp: Any = None,
    package: str | None = None,
    sep: str = ".",
    default: Any = None,
    **kwargs: Any,
) -> Any: ...
def get_full_cls_name(obj: Any) -> str: ...
def import_from_cwd(
    module: str, imp: Any = None, package: str | None = None
) -> Any: ...
def instantiate(name: str, *args: Any, **kwargs: Any) -> Any: ...
def memoize(
    maxsize: int | None = None,
    keyfun: Callable[..., Any] | None = None,
    Cache: type[LRUCache[Any, Any]] = ...,
) -> Callable[[Callable[..., _T]], Callable[..., _T]]: ...
def nodename(name: str, hostname: str) -> str: ...
def nodesplit(name: str) -> tuple[None, str] | list[str]: ...
def noop(*args: Any, **kwargs: Any) -> None: ...
def worker_direct(hostname: str | kombu.Queue) -> kombu.Queue: ...



from typing import Any

import amqp
from kombu.transport.base import Message as BaseMessage
from kombu.transport.base import StdChannel
from kombu.transport.base import Transport as BaseTransport
from typing_extensions import override

DEFAULT_PORT: int
DEFAULT_SSL_PORT: int

class Message(BaseMessage):
    def __init__(self, msg: Any, channel: Any = ..., **kwargs: Any) -> None: ...

_Message = Message

class Channel(amqp.Channel, StdChannel):
    Message: type[_Message]  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleVariableOverride]

    def prepare_message(
        self,
        body: bytes | str,
        priority: int | None = ...,
        content_type: str | None = ...,
        content_encoding: str | None = ...,
        headers: dict[str, Any] | None = ...,
        properties: dict[str, Any] | None = ...,
        _Message: type[amqp.Message] = ...,
    ) -> amqp.Message: ...
    @override
    def prepare_queue_arguments(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, arguments: dict[str, Any], **kwargs: Any
    ) -> dict[str, Any]: ...
    def message_to_python(self, raw_message: Any) -> _Message: ...

_Connection = amqp.Connection

class Connection(amqp.Connection):
    Channel: type[Channel]  # pyright: ignore[reportIncompatibleVariableOverride]

class Transport(BaseTransport):
    Connection: type[_Connection]
    default_port: int  # pyright: ignore[reportIncompatibleVariableOverride]
    default_ssl_port: int
    connection_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    channel_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    recoverable_connection_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    recoverable_channel_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_name: str
    driver_type: str

    def __init__(
        self,
        client: Any,
        default_port: int | None = ...,
        default_ssl_port: int | None = ...,
        **kwargs: Any,
    ) -> None: ...
    @override
    def driver_version(self) -> str: ...
    @override
    def create_channel(self, connection: _Connection) -> Channel: ...
    @override
    def drain_events(self, connection: _Connection, **kwargs: Any) -> Any: ...
    @override
    def establish_connection(self) -> _Connection: ...
    @override
    def verify_connection(self, connection: _Connection) -> bool: ...
    @override
    def close_connection(self, connection: _Connection) -> None: ...
    @override
    def get_heartbeat_interval(self, connection: _Connection) -> int: ...
    @override
    def register_with_event_loop(self, connection: _Connection, loop: Any) -> None: ...
    @override
    def heartbeat_check(self, connection: _Connection, rate: int = ...) -> Any: ...
    @override
    def qos_semantics_matches_spec(self, connection: _Connection) -> bool: ...
    @property
    @override
    def default_connection_params(self) -> dict[str, Any]: ...
    @override
    def get_manager(self, *args: Any, **kwargs: Any) -> Any: ...

class SSLTransport(Transport):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...



from collections.abc import Callable, Iterable
from ctypes import Array as _CTypesArray
from ctypes import _CData
from logging import Logger
from multiprocessing import (
    AuthenticationError as AuthenticationError,
)
from multiprocessing import (
    BufferTooShort as BufferTooShort,
)
from multiprocessing import (
    ProcessError as ProcessError,
)
from multiprocessing import (
    TimeoutError as TimeoutError,
)
from multiprocessing.context import BaseContext
from multiprocessing.managers import SyncManager
from multiprocessing.queues import (
    JoinableQueue as _JoinableQueue,
)
from multiprocessing.queues import (
    Queue as _Queue,
)
from multiprocessing.queues import (
    SimpleQueue as _SimpleQueue,
)
from multiprocessing.sharedctypes import SynchronizedBase
from multiprocessing.synchronize import (
    Barrier as _Barrier,
)
from multiprocessing.synchronize import (
    BoundedSemaphore as _BoundedSemaphore,
)
from multiprocessing.synchronize import (
    Condition as _Condition,
)
from multiprocessing.synchronize import (
    Event as _Event,
)
from multiprocessing.synchronize import (
    Lock as _Lock,
)
from multiprocessing.synchronize import (
    RLock as _RLock,
)
from multiprocessing.synchronize import (
    Semaphore as _Semaphore,
)
from typing import Any

from billiard import einfo as einfo
from billiard import exceptions as exceptions
from billiard.connection import Connection
from billiard.context import Process as Process
from billiard.exceptions import (
    SoftTimeLimitExceeded as SoftTimeLimitExceeded,
)
from billiard.exceptions import (
    TimeLimitExceeded as TimeLimitExceeded,
)
from billiard.exceptions import (
    WorkerLostError as WorkerLostError,
)
from billiard.pool import Pool as _Pool
from billiard.process import active_children as active_children
from billiard.process import current_process as current_process

__all__ = [
    "Array",
    "AuthenticationError",
    "Barrier",
    "BoundedSemaphore",
    "BufferTooShort",
    "Condition",
    "Event",
    "JoinableQueue",
    "Lock",
    "Manager",
    "Pipe",
    "Pool",
    "Process",
    "ProcessError",
    "Queue",
    "RLock",
    "RawArray",
    "RawValue",
    "Semaphore",
    "SimpleQueue",
    "SoftTimeLimitExceeded",
    "TimeLimitExceeded",
    "TimeoutError",
    "Value",
    "WorkerLostError",
    "active_children",
    "allow_connection_pickling",
    "cpu_count",
    "current_process",
    "forking_enable",
    "forking_is_enabled",
    "freeze_support",
    "get_all_start_methods",
    "get_context",
    "get_logger",
    "get_start_method",
    "log_to_stderr",
    "set_executable",
    "set_forkserver_preload",
    "set_start_method",
]

def Array(
    typecode_or_type: str | type[_CData],
    size_or_initializer: int | Iterable[Any],
    *args: Any,
    **kwargs: Any,
) -> _CTypesArray[Any]: ...
def Barrier(
    parties: int, action: Callable[[], Any] | None = ..., timeout: float | None = ...
) -> _Barrier: ...
def BoundedSemaphore(value: int = ...) -> _BoundedSemaphore: ...
def Condition(lock: _Lock | _RLock | None = ...) -> _Condition: ...
def Event() -> _Event: ...
def JoinableQueue(maxsize: int = ...) -> _JoinableQueue[Any]: ...
def Lock() -> _Lock: ...
def Manager() -> SyncManager: ...
def Pipe(
    duplex: bool = ..., rnonblock: bool = ..., wnonblock: bool = ...
) -> tuple[Connection, Connection]: ...
def Pool(
    processes: int | None = ...,
    initializer: Callable[..., Any] | None = ...,
    initargs: tuple[Any, ...] = ...,
    maxtasksperchild: int | None = ...,
    timeout: float | None = ...,
    soft_timeout: float | None = ...,
    lost_worker_timeout: float | None = ...,
    max_restarts: int | None = ...,
    max_restart_freq: int = ...,
    on_process_up: Callable[..., Any] | None = ...,
    on_process_down: Callable[..., Any] | None = ...,
    on_timeout_set: Callable[..., Any] | None = ...,
    on_timeout_cancel: Callable[..., Any] | None = ...,
    threads: bool = ...,
    semaphore: Any | None = ...,
    putlocks: bool = ...,
    allow_restart: bool = ...,
) -> _Pool: ...
def Queue(maxsize: int = ...) -> _Queue[Any]: ...
def RLock() -> _RLock: ...
def RawArray(
    typecode_or_type: str | type[_CData], size_or_initializer: int | Iterable[Any]
) -> _CTypesArray[Any]: ...
def RawValue(typecode_or_type: str | type[_CData], *args: Any) -> Any: ...
def Semaphore(value: int = ...) -> _Semaphore: ...
def SimpleQueue() -> _SimpleQueue[Any]: ...
def Value(
    typecode_or_type: str | type[_CData], *args: Any, **kwargs: Any
) -> SynchronizedBase[Any]: ...
def allow_connection_pickling() -> None: ...
def cpu_count() -> int: ...
def forking_enable(value: bool) -> None: ...
def forking_is_enabled() -> bool: ...
def freeze_support() -> None: ...
def get_all_start_methods() -> list[str]: ...
def get_context(method: str | None = ...) -> BaseContext: ...
def get_logger() -> Logger: ...
def get_start_method(allow_none: bool = ...) -> str | None: ...
def log_to_stderr(level: int | None = ...) -> Logger: ...
def set_executable(executable: str) -> None: ...
def set_forkserver_preload(module_names: list[str]) -> None: ...
def set_start_method(method: str, force: bool = ...) -> None: ...



from builtins import type as _type
from collections.abc import Callable
from typing import Any

__all__ = ("NAMESPACES", "Option", "find", "flatten")

class Option:
    default: Any
    type: str | None
    deprecate_by: str | None
    remove_by: str | None
    alt: tuple[str, ...] | None
    old: set[str]
    typemap: dict[str, _type]

    def __init__(
        self,
        default: Any = None,
        *args: Any,
        **kwargs: Any,
    ) -> None: ...
    def to_python(self, value: Any) -> Any: ...

DEFAULTS: dict[str, dict[str, Any]]
NAMESPACES: dict[str, dict[str, Option]]
SETTING_KEYS: set[str]

def find(name: str, namespace: str = "celery") -> tuple[str, Option] | None: ...
def flatten(
    d: dict[str, dict[str, Any]], root: str = "", keyfilter: Callable[..., Any] = ...
) -> dict[str, Any]: ...
def strtobool(
    term: str | bool | int | None, table: dict[str, bool] | None = None
) -> bool: ...



from typing import Any

def annotation_is_class(annotation: Any) -> bool: ...
def annotation_issubclass(annotation: Any, cls: type) -> bool: ...
def get_optional_arg(annotation: Any) -> Any: ...
def is_none_type(value: Any) -> bool: ...



from typing import Any

from sqlalchemy.orm import RelationshipProperty
from sqlalchemy.schema import Column, MetaData

class_registry: dict[str, Any]
metadata: MetaData
ModelBase: type[Any]

class Queue:
    __table_args__: dict[str, Any]
    __tablename__: str

    id: Column[int]
    name: Column[str]
    messages: RelationshipProperty[Any]

    def __init__(self, name: str) -> None: ...

class Message:
    __table_args__: tuple[Any, ...]
    __tablename__: str
    __mapper_args__: dict[str, Any]

    id: Column[int]
    visible: Column[bool]
    sent_at: Column[Any]
    payload: Column[str]
    version: Column[int]
    queue_id: Column[int]
    queue: Queue

    def __init__(self, payload: str, queue: Queue) -> None: ...



from typing import Any, NamedTuple

from billiard import pool as _pool
from typing_extensions import override

__all__ = ("AsynPool",)

class Ack(NamedTuple):
    id: int
    fd: int
    payload: Any

class Worker(_pool.Worker): ...

class ResultHandler(_pool.ResultHandler):
    @override
    def on_stop_not_started(self) -> None: ...
    def register_with_event_loop(self, hub: Any) -> None: ...

class AsynPool(_pool.Pool):
    @property
    def timers(self) -> dict[Any, Any]: ...
    def __init__(
        self,
        processes: int | None = None,
        synack: bool = False,
        sched_strategy: int | None = None,
        proc_alive_timeout: float | None = None,
        *args: Any,
        **kwargs: Any,
    ) -> None: ...
    def flush(self) -> None: ...
    def create_process_queues(self) -> tuple[Any, Any, Any]: ...
    @override
    def create_result_handler(self) -> ResultHandler: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def destroy_queues(self, queues: tuple[Any, Any, Any], proc: Any) -> None: ...
    def human_write_stats(self) -> str: ...
    @override
    def on_partial_read(self, job: Any, proc: Any) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    def on_process_alive(self, pid: int) -> None: ...
    @override
    def process_flush_queues(self, proc: Any) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    def register_with_event_loop(self, hub: Any) -> None: ...



from collections.abc import Callable

__all__ = ["BaseProcess", "Process", "active_children", "current_process"]

def current_process() -> BaseProcess: ...
def active_children(_cleanup: Callable[[], None] = ...) -> list[BaseProcess]: ...

class BaseProcess:
    name: str
    def __init__(
        self,
        group: None = ...,
        target: Callable[..., object] | None = ...,
        name: str | None = ...,
        args: tuple[object, ...] = ...,
        kwargs: dict[str, object] = ...,
        daemon: bool | None = ...,
        **_kw: object,
    ) -> None: ...
    def run(self) -> None: ...
    def start(self) -> None: ...
    def close(self) -> None: ...
    def terminate(self) -> None: ...
    def terminate_controlled(self) -> None: ...
    def join(self, timeout: int | None = None) -> None: ...
    def is_alive(self) -> bool: ...
    @property
    def daemon(self) -> bool: ...
    @daemon.setter
    def daemon(self, name: bool) -> None: ...
    @property
    def authkey(self) -> AuthenticationString: ...
    @authkey.setter
    def authkey(self, authkey: AuthenticationString) -> None: ...
    @property
    def exitcode(self) -> int | None: ...
    @property
    def ident(self) -> int | None: ...

    pid = ident

    @property
    def sentinel(self) -> int: ...

class AuthenticationString(bytes): ...

class _MainProcess(BaseProcess):
    def __init__(self) -> None: ...

Process = BaseProcess



from typing import Any

from celery.backends.base import KeyValueStoreBackend

__all__ = ("ArangoDbBackend",)

class ArangoDbBackend(KeyValueStoreBackend):
    host: str
    port: str
    database: str
    collection: str
    http_protocol: str
    username: str | None
    password: str | None
    verify: bool
    key_t: type[str]  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleMethodOverride]

    @property
    def connection(self) -> Any: ...
    @property
    def db(self) -> Any: ...
    @property
    def expires_delta(self) -> float: ...
    def __init__(self, url: str | None = None, *args: Any, **kwargs: Any) -> None: ...



from collections.abc import Iterator
from contextlib import contextmanager
from typing import Any

__all__ = ("get_digest_algorithm", "reraise_errors")

def get_digest_algorithm(digest: str = "sha256") -> Any: ...
@contextmanager
def reraise_errors(
    msg: str = "{0!r}", errors: tuple[type[Exception], ...] | None = None
) -> Iterator[None]: ...

class SecurityError(Exception): ...



from collections.abc import Callable
from typing import Any, Generic, TypeVar, overload

__all__ = ("cached_property",)

_T = TypeVar("_T")
_S = TypeVar("_S")

class cached_property(Generic[_T]):
    fget: Callable[[Any], _T] | None
    fset: Callable[[Any, _T], None] | None
    fdel: Callable[[Any], None] | None
    attrname: str | None

    @overload
    def __init__(
        self,
        fget: Callable[[Any], _T],
        fset: Callable[[Any, _T], None] | None = ...,
        fdel: Callable[[Any], None] | None = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: cached_property[Any],
        fget: None = ...,
        fset: Callable[[Any, Any], None] | None = ...,
        fdel: Callable[[Any], None] | None = ...,
    ) -> None: ...
    @overload
    def __get__(
        self, instance: None, owner: type[Any] | None = ...
    ) -> cached_property[_T]: ...
    @overload
    def __get__(self, instance: _S, owner: type[_S] | None = ...) -> _T: ...
    def __set__(self, instance: Any, value: _T) -> None: ...
    def __delete__(self, instance: Any) -> None: ...
    def setter(self, fset: Callable[[Any, _T], None]) -> cached_property[_T]: ...
    def deleter(self, fdel: Callable[[Any], None]) -> cached_property[_T]: ...



import pickle as pickle
from collections.abc import Callable, Container, Iterable, Mapping
from typing import Any, NamedTuple, TypeAlias

__all__ = ("dumps", "loads", "pickle", "register", "unregister")

pickle_load = pickle.load

_Encoder: TypeAlias = Callable[[Any], str]
_Decoder: TypeAlias = Callable[[str], Any]

class codec(NamedTuple):
    content_type: str
    content_encoding: str
    encoder: _Encoder

class SerializerRegistry:
    type_to_name: dict[str, str]
    name_to_type: dict[str, str]
    def __init__(self) -> None: ...
    def register(
        self,
        name: str,
        encoder: _Encoder | None,
        decoder: _Decoder | None,
        content_type: str,
        content_encoding: str = ...,
    ) -> None: ...
    def enable(self, name: str) -> None: ...
    def disable(self, name: str) -> None: ...
    def unregister(self, name: str) -> None: ...
    def _set_default_serializer(self, name: str) -> None: ...
    def _for_untrusted_content(self, ctype: str, why: str) -> bool: ...
    def dumps(
        self, data: Any, serializer: str | None = ...
    ) -> tuple[str, str, str]: ...
    def loads(
        self,
        data: bytes | str,
        content_type: str,
        content_encoding: str,
        accept: Container[str] | None = ...,
        force: bool = ...,
        _trusted_content: Container[str] = ...,
    ) -> Any: ...

def dumps(data: Any, serializer: str | None = ...) -> tuple[str, str, str]: ...
def loads(
    data: bytes | str,
    content_type: str,
    content_encoding: str,
    accept: Container[str] | None = ...,
    force: bool = ...,
    _trusted_content: Container[str] = ...,
) -> Any: ...
def register(
    name: str,
    encoder: _Encoder | None,
    decoder: _Decoder | None,
    content_type: str,
    content_encoding: str = ...,
) -> None: ...
def unregister(name: str) -> None: ...
def raw_encode(data: Any) -> tuple[str, str, str]: ...
def register_json() -> None: ...
def register_yaml() -> None: ...
def register_pickle() -> None: ...
def register_msgpack() -> None: ...
def enable_insecure_serializers(choices: Container[str] | None = ...) -> None: ...
def disable_insecure_serializers(allowed: Container[str] | None = ...) -> None: ...
def prepare_accept_content(
    content_types: Iterable[str] | None, name_to_type: Mapping[str, str] | None = ...
) -> set[str] | None: ...
def _reraise_errors(
    wrapper: type[BaseException],
    include: tuple[type[BaseException], ...] = ...,
    exclude: tuple[type[BaseException], ...] = ...,
) -> Callable[[Callable[..., Any]], Callable[..., Any]]: ...
def pickle_loads(s: bytes, load: Callable[..., Any] = ...) -> Any: ...
def unpickle(s: bytes) -> Any: ...
def parenthesize_alias(first: str, second: str) -> str: ...



import threading
from typing import Any

from billiard.common import (
    TERM_SIGNAL,
)
from billiard.dummy import DummyProcess
from billiard.exceptions import (
    SoftTimeLimitExceeded,
)
from typing_extensions import Self, override

# Note: __all__ is not present at runtime

MAXMEM_USED_FMT: str
SIGKILL = TERM_SIGNAL
RUN: int
CLOSE: int
TERMINATE: int
ACK: int
READY: int
TASK: int
NACK: int
DEATH: int
EX_OK: int
EX_FAILURE: int
EX_RECYCLE: int
LOST_WORKER_TIMEOUT: float
GUARANTEE_MESSAGE_CONSUMPTION_RETRY_LIMIT: int
GUARANTEE_MESSAGE_CONSUMPTION_RETRY_INTERVAL: float
Lock = threading.Lock
PY3: bool
SIG_SOFT_TIMEOUT: int
TIMEOUT_MAX: float

def error(msg: str, *args: Any, **kwargs: Any) -> None: ...

job_counter: Any

def mapstar(args: Any) -> Any: ...
def starmapstar(args: Any) -> Any: ...
def soft_timeout_sighandler(signum: int, frame: Any) -> None: ...
def stop_if_not_current(thread: Any, timeout: float | None = ...) -> None: ...

class LaxBoundedSemaphore(threading.Semaphore):
    def __init__(self, value: int = ..., verbose: Any = ...) -> None: ...
    def shrink(self) -> None: ...
    def grow(self) -> None: ...
    @override
    def release(self) -> None: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def clear(self) -> None: ...

class MaybeEncodingError(Exception):
    def __init__(self, exc: object, value: object) -> None: ...

class WorkersJoined(Exception): ...

class Worker:
    def __init__(
        self,
        inq: Any,
        outq: Any,
        synq: Any = ...,
        initializer: Any = ...,
        initargs: tuple[Any, ...] = ...,
        maxtasks: int | None = ...,
        sentinel: Any = ...,
        on_exit: Any = ...,
        sigprotection: bool = ...,
        wrap_exception: bool = ...,
        max_memory_per_child: int | None = ...,
        on_ready_counter: Any = ...,
    ) -> None: ...
    def __call__(self) -> None: ...
    def after_fork(self) -> None: ...
    def contribute_to_object(self, obj: Any) -> None: ...
    def on_loop_start(self, pid: int) -> None: ...
    def prepare_result(self, result: Any) -> Any: ...
    def workloop(
        self, debug: Any = ..., now: Any = ..., pid: int | None = ...
    ) -> None: ...

class PoolThread(DummyProcess):
    daemon: bool
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def start(self, *args: Any, **kwargs: Any) -> None: ...
    def stop(self, timeout: float | None = ...) -> None: ...
    def on_stop_not_started(self) -> None: ...
    def terminate(self) -> None: ...
    def close(self) -> None: ...

class Supervisor(PoolThread):
    def __init__(self, pool: Any) -> None: ...
    def body(self) -> None: ...

class TaskHandler(PoolThread):
    def __init__(
        self, taskqueue: Any, put: Any, outqueue: Any, pool: Any, cache: Any
    ) -> None: ...
    def body(self) -> None: ...
    def tell_others(self) -> None: ...
    @override
    def on_stop_not_started(self) -> None: ...

class TimeoutHandler(PoolThread):
    def __init__(
        self, processes: Any, cache: Any, t_soft: float | None, t_hard: float | None
    ) -> None: ...
    def body(self) -> None: ...
    def handle_event(self, *args: Any) -> None: ...
    def handle_timeouts(self) -> None: ...
    def on_soft_timeout(self, job: Any) -> None: ...
    def on_hard_timeout(self, job: Any) -> None: ...

class ResultHandler(PoolThread):
    def __init__(
        self,
        outqueue: Any,
        get: Any,
        cache: Any,
        poll: Any,
        join_exited_workers: Any,
        putlock: Any,
        restart_state: Any,
        check_timeouts: Any,
        on_job_ready: Any,
        on_ready_counters: Any = ...,
    ) -> None: ...
    @override
    def on_stop_not_started(self) -> None: ...
    def body(self) -> None: ...
    def handle_event(self, fileno: int | None = ..., events: Any = ...) -> None: ...
    def finish_at_shutdown(self, handle_timeouts: bool = ...) -> None: ...

class Pool:
    def Process(self, *args: Any, **kwds: Any) -> Any: ...
    Worker: type[Worker]
    def WorkerProcess(self, worker: Any) -> Any: ...
    Supervisor: type[Supervisor]
    TaskHandler: type[TaskHandler]
    TimeoutHandler: type[TimeoutHandler]
    ResultHandler: type[ResultHandler]
    SoftTimeLimitExceeded: type[SoftTimeLimitExceeded]

    def __init__(
        self,
        processes: int | None = ...,
        initializer: Any = ...,
        initargs: tuple[Any, ...] = ...,
        maxtasksperchild: int | None = ...,
        timeout: float | None = ...,
        soft_timeout: float | None = ...,
        lost_worker_timeout: float | None = ...,
        max_restarts: int | None = ...,
        max_restart_freq: int = ...,
        on_process_up: Any = ...,
        on_process_down: Any = ...,
        on_timeout_set: Any = ...,
        on_timeout_cancel: Any = ...,
        threads: bool = ...,
        semaphore: Any = ...,
        putlocks: bool = ...,
        allow_restart: bool = ...,
        synack: bool = ...,
        on_process_exit: Any = ...,
        context: Any = ...,
        max_memory_per_child: int | None = ...,
        enable_timeouts: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *exc_info: object) -> None: ...
    def shrink(self, n: int = ...) -> None: ...
    def grow(self, n: int = ...) -> None: ...
    def maintain_pool(self) -> None: ...
    @override
    def __reduce__(self) -> str | tuple[Any, ...]: ...
    def close(self) -> None: ...
    def terminate(self) -> None: ...
    def join(self) -> None: ...
    def restart(self) -> None: ...
    def cpu_count(self) -> int: ...
    def create_result_handler(self, **extra_kwargs: Any) -> Any: ...
    def did_start_ok(self) -> bool: ...
    def get_process_queues(self) -> Any: ...
    def handle_result_event(self, *args: Any) -> None: ...
    def apply(
        self, func: Any, args: tuple[Any, ...] = ..., kwds: dict[str, Any] = ...
    ) -> Any: ...
    def apply_async(
        self,
        func: Any,
        args: tuple[Any, ...] = ...,
        kwds: dict[str, Any] = ...,
        callback: Any = ...,
        error_callback: Any = ...,
        accept_callback: Any = ...,
        timeout_callback: Any = ...,
        waitforslot: bool | None = ...,
        soft_timeout: float | None = ...,
        timeout: float | None = ...,
        lost_worker_timeout: float | None = ...,
        callbacks_propagate: tuple[Any, ...] = ...,
        correlation_id: Any = ...,
    ) -> ApplyResult: ...
    def imap(
        self,
        func: Any,
        iterable: Any,
        chunksize: int = ...,
        lost_worker_timeout: float | None = ...,
    ) -> IMapIterator: ...
    def imap_unordered(
        self,
        func: Any,
        iterable: Any,
        chunksize: int = ...,
        lost_worker_timeout: float | None = ...,
    ) -> IMapUnorderedIterator: ...
    def map(
        self, func: Any, iterable: Any, chunksize: int | None = ...
    ) -> list[Any]: ...
    def map_async(
        self,
        func: Any,
        iterable: Any,
        chunksize: int | None = ...,
        callback: Any = ...,
        error_callback: Any = ...,
    ) -> MapResult: ...
    def starmap(
        self, func: Any, iterable: Any, chunksize: int | None = ...
    ) -> list[Any]: ...
    def starmap_async(
        self,
        func: Any,
        iterable: Any,
        chunksize: int | None = ...,
        callback: Any = ...,
        error_callback: Any = ...,
    ) -> MapResult: ...
    def mark_as_worker_lost(self, job: Any, exitcode: int) -> None: ...
    def on_grow(self, n: int) -> None: ...
    def on_job_process_down(self, job: Any, pid_gone: bool) -> None: ...
    def on_job_process_lost(self, job: Any, pid: int, exitcode: int) -> None: ...
    def on_job_ready(self, job: Any, i: int, obj: Any, inqW_fd: Any) -> None: ...
    def on_partial_read(self, job: Any, worker: Any) -> None: ...
    def on_shrink(self, n: int) -> None: ...
    def process_flush_queues(self, worker: Any) -> None: ...
    @property
    def process_sentinels(self) -> list[Any]: ...
    def send_ack(self, response: Any, job: Any, i: int, fd: Any) -> None: ...
    def terminate_job(self, pid: int, sig: int | None = ...) -> None: ...

class ApplyResult:
    def __init__(
        self,
        cache: Any,
        callback: Any,
        accept_callback: Any = ...,
        timeout_callback: Any = ...,
        error_callback: Any = ...,
        soft_timeout: float | None = ...,
        timeout: float | None = ...,
        lost_worker_timeout: float = ...,
        on_timeout_set: Any = ...,
        on_timeout_cancel: Any = ...,
        callbacks_propagate: tuple[Any, ...] = ...,
        send_ack: Any = ...,
        correlation_id: Any = ...,
    ) -> None: ...
    def ready(self) -> bool: ...
    def accepted(self) -> bool: ...
    def successful(self) -> bool: ...
    def get(self, timeout: float | None = ...) -> Any: ...
    def wait(self, timeout: float | None = ...) -> bool: ...
    def terminate(self, signum: int) -> None: ...
    def worker_pids(self) -> list[int]: ...
    def safe_apply_callback(self, fun: Any, *args: Any, **kwargs: Any) -> None: ...
    def discard(self) -> None: ...
    def handle_timeout(self, soft: bool = ...) -> None: ...

class MapResult(ApplyResult):
    def __init__(
        self,
        cache: Any,
        chunksize: int,
        length: int,
        callback: Any,
        error_callback: Any,
    ) -> None: ...

class IMapIterator:
    def __init__(self, cache: Any, lost_worker_timeout: float = ...) -> None: ...
    def __iter__(self) -> IMapIterator: ...
    def __next__(self, timeout: float | None = ...) -> Any: ...
    def next(self, timeout: float | None = ...) -> Any: ...
    def ready(self) -> bool: ...
    def worker_pids(self) -> list[int]: ...

class IMapUnorderedIterator(IMapIterator): ...

class ThreadPool(Pool):
    Process = DummyProcess  # pyright: ignore[reportAssignmentType]
    DummyProcess: type[DummyProcess]
    def __init__(
        self,
        processes: int | None = ...,
        initializer: Any = ...,
        initargs: tuple[Any, ...] = ...,
    ) -> None: ...



from queue import Empty as Empty
from types import TracebackType
from typing import Any

from kombu.entity import Exchange, Queue
from kombu.message import Message
from kombu.messaging import Consumer, Producer
from kombu.transport.base import StdChannel
from typing_extensions import Self

__all__ = ("SimpleBuffer", "SimpleQueue")

class SimpleBase:
    Empty: type[Empty]

    channel: StdChannel
    producer: Producer
    consumer: Consumer
    queue: Queue
    exchange: Exchange
    no_ack: bool

    def __init__(
        self,
        channel: StdChannel,
        producer: Producer,
        consumer: Consumer,
        no_ack: bool = ...,
    ) -> None: ...
    def get(self, block: bool = ..., timeout: float | None = ...) -> Message: ...
    def get_nowait(self) -> Message: ...
    def put(
        self,
        message: Any,
        serializer: str | None = ...,
        headers: dict[str, Any] | None = ...,
        compression: str | None = ...,
        routing_key: str | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def clear(self) -> int: ...
    def qsize(self) -> int: ...
    def close(self) -> None: ...
    def _consume(self) -> None: ...
    def _receive(self, message_data: Any, message: Message) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    @property
    def __name__(self) -> str: ...

class SimpleQueue(SimpleBase):
    queue_opts: dict[str, Any]
    queue_args: dict[str, Any]
    exchange_opts: dict[str, Any]

    def __init__(
        self,
        channel: StdChannel,
        name: str,
        no_ack: bool | None = ...,
        queue_opts: dict[str, Any] | None = ...,
        queue_args: dict[str, Any] | None = ...,
        exchange_opts: dict[str, Any] | None = ...,
        serializer: str | None = ...,
        compression: str | None = ...,
        accept: list[str] | None = ...,
    ) -> None: ...

class SimpleBuffer(SimpleQueue):
    queue_opts: dict[str, Any]
    exchange_opts: dict[str, Any]



import types
from typing import Any, TypeAlias

from kombu.connection import Connection
from kombu.message import Message as Message
from kombu.messaging import Consumer, Producer
from kombu.utils.objects import cached_property
from typing_extensions import Self, override

__all__ = ("Management", "Message", "StdChannel", "Transport")

class Implements(dict[str, Any]):
    def __getattr__(self, key: str) -> Any: ...
    @override
    def __setattr__(self, key: str, value: Any) -> None: ...
    def extend(self, **kwargs: Any) -> Implements: ...

# Forward reference for Management to avoid name collision with Transport.Management
_ManagementType: TypeAlias = Management

class Management:
    transport: Transport
    def __init__(self, transport: Transport) -> None: ...
    def get_bindings(self) -> list[dict[str, Any]]: ...

class Transport:
    Management: type[Management]

    client: Connection | None
    can_parse_url: bool
    default_port: int | None
    connection_errors: tuple[type[BaseException], ...]
    channel_errors: tuple[type[BaseException], ...]
    driver_type: str
    driver_name: str
    recoverable_connection_errors: tuple[type[BaseException], ...]
    recoverable_channel_errors: tuple[type[BaseException], ...]
    implements: Implements

    def __init__(self, client: Connection, **kwargs: Any) -> None: ...
    def establish_connection(self) -> Any: ...
    def close_connection(self, connection: Any) -> None: ...
    def create_channel(self, connection: Any) -> StdChannel: ...
    def close_channel(self, connection: Any) -> None: ...
    def drain_events(self, connection: Any, **kwargs: Any) -> None: ...
    def heartbeat_check(self, connection: Any, rate: int = ...) -> None: ...
    def driver_version(self) -> str: ...
    def get_heartbeat_interval(self, connection: Any) -> int: ...
    def verify_connection(self, connection: Any) -> bool: ...
    def qos_semantics_matches_spec(self, connection: Any) -> bool: ...
    def on_readable(self, connection: Any, loop: Any) -> None: ...
    def as_uri(
        self,
        uri: str,
        include_password: bool = ...,
        mask: str = ...,
    ) -> str: ...
    def get_manager(self, *args: Any, **kwargs: Any) -> _ManagementType: ...
    def register_with_event_loop(self, connection: Any, loop: Any) -> None: ...
    def unregister_from_event_loop(self, connection: Any, loop: Any) -> None: ...
    @property
    def default_connection_params(self) -> dict[str, Any]: ...
    @cached_property
    def manager(self) -> _ManagementType: ...
    @property
    def supports_heartbeats(self) -> bool: ...
    @property
    def supports_ev(self) -> bool: ...

class StdChannel:
    no_ack_consumers: set[str] | None

    def Consumer(self, *args: Any, **kwargs: Any) -> Consumer: ...  # ty: ignore[invalid-type-form]
    def Producer(self, *args: Any, **kwargs: Any) -> Producer: ...  # ty: ignore[invalid-type-form]
    def get_bindings(self) -> list[dict[str, Any]]: ...
    def after_reply_message_received(self, queue: str) -> None: ...
    def prepare_queue_arguments(
        self, arguments: dict[str, Any] | None, **kwargs: Any
    ) -> dict[str, Any] | None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: types.TracebackType | None,
    ) -> None: ...



from typing import Any

__all__ = ("MapAnnotation", "prepare", "resolve_all")

class MapAnnotation:
    def __init__(self, d: dict[str, Any]) -> None: ...
    def annotate(self, task: Any) -> dict[str, Any] | None: ...
    def annotate_any(self) -> dict[str, Any] | None: ...

def prepare(annotations: Any) -> Any: ...
def resolve_all(anno: Any, task: Any) -> dict[str, Any]: ...



from typing import Any

from celery.backends.base import Backend
from typing_extensions import override

__all__ = ("MongoBackend",)

class MongoBackend(Backend):
    host: str
    port: int
    database_name: str
    taskmeta_collection: str
    groupmeta_collection: str
    options: dict[str, Any] | None
    mongo_host: str | None
    max_pool_size: int
    user: str | None
    password: str | None

    @property
    def collection(self) -> Any: ...
    @property
    def database(self) -> Any: ...
    @property
    def group_collection(self) -> Any: ...
    @property
    def expires_delta(self) -> float: ...
    def __init__(self, app: Any = None, **kwargs: Any) -> None: ...
    @override
    def decode(self, data: Any) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]



from collections.abc import Callable, Sequence
from typing import Any

from kombu.connection import Connection
from kombu.message import Message
from kombu.messaging import Consumer as MessagingConsumer
from kombu.messaging import Producer
from kombu.transport.base import StdChannel
from kombu.utils.limits import TokenBucket
from typing_extensions import override

__all__ = ("ConsumerMixin", "ConsumerProducerMixin")

class ConsumerMixin:
    connect_max_retries: int | None
    should_stop: bool
    def get_consumers(
        self, Consumer: Callable[..., MessagingConsumer], channel: StdChannel
    ) -> Sequence[MessagingConsumer]: ...
    def on_connection_revived(self) -> None: ...
    def on_consume_ready(
        self,
        connection: Connection,
        channel: StdChannel,
        consumers: Sequence[MessagingConsumer],
        **kwargs: Any,
    ) -> None: ...
    def on_consume_end(self, connection: Connection, channel: StdChannel) -> None: ...
    def on_iteration(self) -> None: ...
    def on_decode_error(self, message: Message, exc: Exception) -> None: ...
    def on_connection_error(self, exc: Exception, interval: int) -> None: ...
    def extra_context(self, connection: Connection, channel: StdChannel) -> None: ...
    def run(self, _tokens: int = ..., **kwargs: Any) -> None: ...
    def consumer_context(
        self, **kwargs: Any
    ) -> tuple[Connection, StdChannel, Sequence[MessagingConsumer]]: ...
    def consume(
        self,
        limit: int | None = ...,
        timeout: int | None = ...,
        safety_interval: int = ...,
        **kwargs: Any,
    ) -> None: ...
    def maybe_conn_error(self, fun: Callable[..., Any] | None) -> Any: ...
    def create_connection(self) -> Connection: ...
    def establish_connection(self) -> Connection: ...
    def _consume_from(self, *queues: Any) -> Sequence[MessagingConsumer]: ...
    def Consumer(
        self,
    ) -> tuple[Connection, StdChannel, Sequence[MessagingConsumer]]: ...
    @property
    def restart_limit(self) -> TokenBucket: ...
    @property
    def connection_errors(self) -> Sequence[Exception]: ...
    @property
    def channel_errors(self) -> Sequence[Exception]: ...

class ConsumerProducerMixin(ConsumerMixin):
    @override
    def on_consume_end(self, connection: Connection, channel: StdChannel) -> None: ...
    @property
    def producer(self) -> Producer: ...
    @property
    def producer_connection(self) -> Connection: ...



from collections.abc import Iterable, Iterator
from typing import IO, Any

from typing_extensions import override

__all__ = ("DOT", "CycleError", "DependencyGraph", "GraphFormatter")

class DOT:
    ATTR: str
    ATTRSEP: str
    DIRS: dict[str, str]
    EDGE: str
    HEAD: str
    NODE: str
    TAIL: str

class CycleError(Exception):
    def __init__(self, node: Any) -> None: ...

class GraphFormatter:
    edge_scheme: dict[str, Any]
    graph_scheme: dict[str, Any]
    node_scheme: dict[str, Any]
    scheme: dict[str, Any]
    term_scheme: dict[str, Any]

    def __init__(
        self,
        root: Any | None = None,
        type: str | None = None,
        id: str | None = None,
        indent: int = 0,
        inw: str = "    ",
        **scheme: Any,
    ) -> None: ...
    def attr(self, name: str, value: Any) -> str: ...
    def attrs(self, d: dict[str, Any], scheme: dict[str, Any] | None = None) -> str: ...
    def draw_edge(
        self,
        a: Any,
        b: Any,
        scheme: dict[str, Any] | None = None,
        attrs: dict[str, Any] | None = None,
    ) -> str: ...
    def draw_node(
        self,
        obj: Any,
        scheme: dict[str, Any] | None = None,
        attrs: dict[str, Any] | None = None,
    ) -> str: ...
    def edge(self, a: Any, b: Any, **attrs: Any) -> str: ...
    def FMT(self, fmt: str, *args: Any, **kwargs: Any) -> str: ...
    def head(self, **attrs: Any) -> str: ...
    def label(self, obj: Any) -> str: ...
    def node(self, obj: Any, **attrs: Any) -> str: ...
    def tail(self) -> str: ...
    def terminal_node(self, obj: Any, **attrs: Any) -> str: ...

class DependencyGraph(dict[Any, Any]):
    def __init__(
        self, it: Iterable[Any] | None = None, formatter: GraphFormatter | None = None
    ) -> None: ...
    def add_arc(self, obj: Any) -> None: ...
    def add_edge(self, A: Any, B: Any) -> None: ...
    def connect(self, graph: DependencyGraph) -> None: ...
    def edges(self) -> list[tuple[Any, Any]]: ...
    def format(self, obj: Any) -> str: ...
    def iteritems(self) -> Iterator[tuple[Any, Any]]: ...
    def repr_node(self, obj: Any, level: int = 1, fmt: str = "{0}({1})") -> str: ...
    def to_dot(self, fh: IO[str], formatter: GraphFormatter | None = None) -> None: ...
    def topsort(self) -> list[Any]: ...
    @override
    def update(self, it: Iterable[Any]) -> None: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def valency_of(self, obj: Any) -> int: ...



from typing import Any

from celery.backends.base import KeyValueStoreBackend

__all__ = ("ElasticsearchBackend",)

class ElasticsearchBackend(KeyValueStoreBackend):
    doc_type: str | None
    es_max_retries: int
    es_retry_on_timeout: bool
    es_timeout: int
    host: str
    port: int
    index: str
    scheme: str
    username: str | None
    password: str | None

    @property
    def server(self) -> str: ...
    def __init__(self, url: str | None = None, *args: Any, **kwargs: Any) -> None: ...



from collections.abc import Callable, Generator, Iterable, Mapping
from contextlib import contextmanager
from typing import Any, BinaryIO, TextIO
from uuid import UUID

from kombu.utils.functional import reprkwargs as reprkwargs
from kombu.utils.functional import retry_over_time as retry_over_time
from kombu.utils.objects import cached_property as cached_property

# Note: runtime __all__ includes reprkwargs but it's not actually imported in kombu (kombu bug)
# We include it here to match runtime __all__, stubtest allowlist handles the missing definition
__all__ = (
    "EqualityDict",
    "cached_property",
    "emergency_dump_state",
    "fileno",
    "fxrange",
    "fxrangemax",
    "maybe_fileno",
    "maybe_list",
    "nested",
    "register_after_fork",
    "reprcall",
    "reprkwargs",
    "retry_over_time",
    "symbol_by_name",
    "uuid",
)

class EqualityDict(dict[Any, Any]): ...

def uuid(_uuid: Callable[[], UUID] = ...) -> str: ...
def maybe_list(obj: Any, scalars: tuple[type[Any], ...] = ...) -> list[Any] | None: ...
def fxrange(
    start: float = ...,
    stop: float | None = ...,
    step: float = ...,
    repeatlast: bool = ...,
) -> Generator[float, None, None]: ...
def fxrangemax(
    start: float = ...,
    stop: float | None = ...,
    step: float = ...,
    max: float = ...,
) -> Generator[float, None, None]: ...
def emergency_dump_state(
    state: Any,
    open_file: Callable[..., BinaryIO | TextIO] = ...,
    dump: Callable[..., None] | None = ...,
    stderr: TextIO | None = ...,
) -> None: ...
def register_after_fork(obj: Any, func: Callable[..., Any]) -> None: ...
def reprcall(
    name: str,
    args: Iterable[Any] = ...,
    kwargs: Mapping[str, Any] | None = ...,
    sep: str = ...,
) -> str: ...
def symbol_by_name(
    name: str,
    aliases: Mapping[str, str] | None = ...,
    imp: Callable[[str], Any] | None = ...,
    package: str | None = ...,
    sep: str = ...,
    default: Any = ...,
    **kwargs: Any,
) -> Any: ...
@contextmanager
def nested(*managers: Any) -> Generator[tuple[Any, ...], None, None]: ...
def fileno(f: Any) -> int: ...
def maybe_fileno(f: Any) -> int | None: ...



from collections.abc import Callable, Mapping, Sequence
from typing import Any

from kombu.abstract import MaybeChannelBound, Object
from kombu.message import Message as _Message
from kombu.transport.base import StdChannel
from typing_extensions import override

__all__ = ("Exchange", "Queue", "binding", "maybe_delivery_mode")

def _reprstr(s: str) -> str: ...
def pretty_bindings(bindings: Sequence[binding]) -> str: ...
def maybe_delivery_mode(
    v: Any, modes: Mapping[str, int] | None = ..., default: int = ...
) -> int | None: ...

class Exchange(MaybeChannelBound):
    PERSISTENT_DELIVERY_MODE: int
    TRANSIENT_DELIVERY_MODE: int

    name: str
    type: str
    durable: bool
    auto_delete: bool
    delivery_mode: int | None
    no_declare: bool
    passive: bool
    arguments: dict[str, Any] | None

    def __init__(
        self,
        name: str = ...,
        type: str = ...,
        channel: StdChannel | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def Message(
        self,
        body: Any,
        delivery_mode: int | None = ...,
        properties: dict[str, Any] | None = ...,
        **kwargs: Any,
    ) -> _Message: ...
    def declare(
        self,
        nowait: bool = ...,
        passive: bool | None = ...,
        channel: StdChannel | None = ...,
    ) -> None: ...
    def delete(self, if_unused: bool = ..., nowait: bool = ...) -> None: ...
    def bind_to(
        self,
        exchange: str | Exchange = ...,
        routing_key: str = ...,
        arguments: dict[str, Any] | None = ...,
        nowait: bool = ...,
        channel: StdChannel | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def unbind_from(
        self,
        source: str | Exchange = ...,
        routing_key: str = ...,
        nowait: bool = ...,
        arguments: dict[str, Any] | None = ...,
        channel: StdChannel | None = ...,
    ) -> None: ...
    def publish(
        self,
        message: Any,
        routing_key: str | None = ...,
        mandatory: bool = ...,
        immediate: bool = ...,
        exchange: str | None = ...,
    ) -> None: ...
    def binding(
        self,
        routing_key: str = ...,
        arguments: dict[str, Any] | None = ...,
        unbind_arguments: dict[str, Any] | None = ...,
    ) -> binding: ...  # ty: ignore[invalid-type-form]
    def _can_declare(self) -> bool: ...
    @override
    def __eq__(self, other: object) -> bool: ...
    @override
    def __ne__(self, other: object) -> bool: ...
    @override
    def __hash__(self) -> int: ...

class Queue(MaybeChannelBound):
    ContentDisallowed: type[Exception]

    name: str
    exchange: Exchange
    routing_key: str
    durable: bool
    exclusive: bool
    auto_delete: bool
    no_ack: bool
    queue_arguments: dict[str, Any] | None
    binding_arguments: dict[str, Any] | None
    consumer_arguments: dict[str, Any] | None
    bindings: Sequence[binding]
    on_declared: Callable[..., Any] | None

    def __init__(
        self,
        name: str = ...,
        exchange: Exchange | str | None = ...,
        routing_key: str = ...,
        channel: StdChannel | None = ...,
        bindings: Sequence[binding] | None = ...,
        on_declared: Callable[..., Any] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    @classmethod
    def from_dict(cls, queue: str, **options: Any) -> Queue: ...
    def declare(
        self, nowait: bool = ..., channel: StdChannel | None = ...
    ) -> str | None: ...
    def queue_declare(
        self, nowait: bool = ..., passive: bool = ..., channel: StdChannel | None = ...
    ) -> str | None: ...
    def queue_bind(
        self, nowait: bool = ..., channel: StdChannel | None = ...
    ) -> None: ...
    def queue_unbind(
        self,
        arguments: dict[str, Any] | None = ...,
        nowait: bool = ...,
        channel: StdChannel | None = ...,
    ) -> None: ...
    def delete(
        self,
        if_unused: bool = ...,
        if_empty: bool = ...,
        nowait: bool = ...,
    ) -> int | None: ...
    def bind_to(
        self,
        exchange: str | Exchange = ...,
        routing_key: str = ...,
        arguments: dict[str, Any] | None = ...,
        nowait: bool = ...,
        channel: StdChannel | None = ...,
    ) -> None: ...
    def unbind_from(
        self,
        exchange: str | Exchange = ...,
        routing_key: str = ...,
        arguments: dict[str, Any] | None = ...,
        nowait: bool = ...,
        channel: StdChannel | None = ...,
    ) -> None: ...
    def get(
        self, no_ack: bool | None = ..., accept: Sequence[str] | None = ...
    ) -> _Message | None: ...
    def purge(self, nowait: bool = ...) -> int | None: ...
    def consume(
        self,
        consumer_tag: str = ...,
        callback: Callable[[Any, _Message], None] | None = ...,
        no_ack: bool | None = ...,
        nowait: bool = ...,
        on_cancel: Callable[[str], None] | None = ...,
    ) -> str: ...
    def cancel(self, consumer_tag: str) -> None: ...
    def _create_exchange(
        self, nowait: bool = ..., channel: StdChannel | None = ...
    ) -> None: ...
    def _create_queue(
        self, nowait: bool = ..., channel: StdChannel | None = ...
    ) -> str | None: ...
    def _create_bindings(
        self, nowait: bool = ..., channel: StdChannel | None = ...
    ) -> None: ...
    @override
    def __eq__(self, other: object) -> bool: ...
    @override
    def __ne__(self, other: object) -> bool: ...
    @override
    def __hash__(self) -> int: ...

class binding(Object):
    exchange: Exchange | None
    routing_key: str
    arguments: dict[str, Any] | None
    unbind_arguments: dict[str, Any] | None

    def __init__(
        self,
        exchange: Exchange | None = ...,
        routing_key: str = ...,
        arguments: dict[str, Any] | None = ...,
        unbind_arguments: dict[str, Any] | None = ...,
    ) -> None: ...
    def declare(self, channel: StdChannel, nowait: bool = ...) -> None: ...
    def bind(
        self,
        entity: Exchange | Queue,
        nowait: bool = ...,
        channel: StdChannel | None = ...,
    ) -> None: ...
    def unbind(
        self,
        entity: Exchange | Queue,
        nowait: bool = ...,
        channel: StdChannel | None = ...,
    ) -> None: ...



from typing import Any

__all__ = ("asynloop", "synloop")

def asynloop(
    obj: Any,
    connection: Any,
    consumer: Any,
    blueprint: Any,
    hub: Any,
    qos: Any,
    heartbeat: float,
    clock: Any,
    hbrate: float = 2.0,
) -> None: ...
def synloop(
    obj: Any,
    connection: Any,
    consumer: Any,
    blueprint: Any,
    hub: Any,
    qos: Any,
    heartbeat: float,
    clock: Any,
    hbrate: float = 2.0,
    **kwargs: Any,
) -> None: ...



from typing import Any

from celery.backends.base import Backend

__all__ = ("by_name", "by_url")

BACKEND_ALIASES: dict[str, str]
UNKNOWN_BACKEND: str

def by_name(
    backend: str | None = None,
    loader: Any | None = None,
    extension_namespace: str = "celery.result_backends",
) -> type[Backend]: ...
def by_url(backend: str | None = None, loader: Any | None = None) -> type[Backend]: ...



from collections.abc import Callable, MutableMapping
from datetime import datetime
from typing import Any

import click
from celery.app.base import Celery
from celery.utils.dispatch.signal import Signal
from kombu.utils.objects import cached_property

class CeleryOption(click.Option): ...

class CeleryCommand(click.Command):
    app: Celery | None
    def __init__(
        self,
        name: str | None,
        context_settings: MutableMapping[str, Any] | None = None,
        callback: Callable[..., Any] | None = None,
        params: list[click.Parameter] | None = None,
        help: str | None = None,
        epilog: str | None = None,
        short_help: str | None = None,
        options_metavar: str | None = "[OPTIONS]",
        add_help_option: bool = True,
        no_args_is_help: bool = False,
        hidden: bool = False,
        deprecated: bool = False,
    ) -> None: ...

class CeleryDaemonCommand(CeleryCommand): ...

class DaemonOption(click.Option):
    def daemon_setting(
        self, ctx: click.Context, opt: CeleryOption, value: Any
    ) -> Any: ...

class CLIContext:
    app: Celery
    OK: cached_property[str]
    ERROR: cached_property[str]

    def __init__(
        self,
        app: Celery,
        no_color: bool,
        workdir: str | None,
        quiet: bool = False,
    ) -> None: ...
    def echo(self, message: str | None = None, **kwargs: Any) -> None: ...
    def error(self, message: str | None = None, **kwargs: Any) -> None: ...
    def pretty(self, n: Any) -> str: ...
    def pretty_dict_ok_error(self, n: dict[str, Any]) -> str: ...
    def pretty_list(self, n: list[Any]) -> str: ...
    def say_chat(
        self, direction: str, title: str, body: str = "", show_body: bool = False
    ) -> None: ...
    def secho(self, message: str | None = None, **kwargs: Any) -> None: ...
    def style(self, message: str | None = None, **kwargs: Any) -> str: ...

class CommaSeparatedList(click.ParamType): ...
class LogLevel(click.ParamType): ...
class ISO8601DateTime(click.ParamType): ...
class ISO8601DateTimeOrFloat(click.ParamType): ...
class JsonArray(click.ParamType): ...
class JsonObject(click.ParamType): ...

COMMA_SEPARATED_LIST: CommaSeparatedList
LOG_LEVEL: LogLevel
ISO8601: ISO8601DateTime
ISO8601_OR_FLOAT: ISO8601DateTimeOrFloat
JSON_ARRAY: JsonArray
JSON_OBJECT: JsonObject
FORMATTER: Any  # pygments Terminal256Formatter
LEXER: Any  # pygments PythonLexer

def handle_preload_options(f: Callable[..., Any]) -> Callable[..., Any]: ...
def maybe_iso8601(dt: datetime | str | None) -> datetime | None: ...
def get_current_app() -> Celery: ...
def mlevel(level: str | int) -> int: ...
def pformat(
    object: Any,
    indent: int = 1,
    width: int = 80,
    depth: int | None = None,
    *,
    compact: bool = False,
    sort_dicts: bool = True,
    underscore_numbers: bool = False,
) -> str: ...

user_preload_options: Signal



from typing import Any

__all__ = ("Heart",)

class Heart:
    def __init__(
        self,
        timer: Any,
        eventer: Any,
        interval: float | None = None,
    ) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...



from typing import Any

__all__ = ("BasePool", "apply_target")

def apply_target(
    target: Any,
    args: Any = ...,
    kwargs: Any | None = ...,
    callback: Any | None = ...,
    accept_callback: Any | None = ...,
    pid: Any | None = ...,
    getpid: Any = ...,
    propagate: Any = ...,
    monotonic: Any = ...,
    **_: Any,
) -> None: ...

class BasePool:
    RUN: int
    CLOSE: int
    TERMINATE: int
    Timer: Any
    signal_safe: bool
    is_green: bool
    uses_semaphore: bool
    task_join_will_block: bool
    body_can_be_buffer: bool
    limit: Any
    putlocks: Any
    options: Any
    forking_enable: Any
    callbacks_propagate: Any
    app: Any
    def __init__(
        self,
        limit: Any | None = ...,
        putlocks: bool = ...,
        forking_enable: bool = ...,
        callbacks_propagate: Any = ...,
        app: Any | None = ...,
        **options: Any,
    ) -> None: ...
    def on_start(self) -> None: ...
    def did_start_ok(self) -> Any: ...
    def flush(self) -> None: ...
    def on_stop(self) -> None: ...
    def register_with_event_loop(self, loop: Any) -> None: ...
    def on_apply(self, *args: Any, **kwargs: Any) -> None: ...
    def on_terminate(self) -> None: ...
    def on_soft_timeout(self, job: Any) -> None: ...
    def on_hard_timeout(self, job: Any) -> None: ...
    def maintain_pool(self, *args: Any, **kwargs: Any) -> None: ...
    def terminate_job(self, pid: Any, signal: Any | None = ...) -> None: ...
    def restart(self) -> None: ...
    def stop(self) -> None: ...
    def terminate(self) -> None: ...
    def start(self) -> None: ...
    def close(self) -> None: ...
    def on_close(self) -> None: ...
    def apply_async(
        self,
        target: Any,
        args: Any | None = ...,
        kwargs: Any | None = ...,
        **options: Any,
    ) -> Any: ...
    @property
    def info(self) -> Any: ...
    @property
    def active(self) -> Any: ...
    @property
    def num_processes(self) -> Any: ...



from typing import Any

from celery.backends.base import KeyValueStoreBackend

__all__ = ("ConsulBackend",)

class ConsulBackend(KeyValueStoreBackend):
    consistency: str
    path: str | None
    consul: Any

    @property
    def client(self) -> Any: ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...



from collections.abc import Callable, Iterable
from ctypes import Array as _CTypesArray
from ctypes import _CData
from logging import Logger
from multiprocessing import (
    AuthenticationError,
    BufferTooShort,
    ProcessError,
    TimeoutError,
)
from multiprocessing.managers import SyncManager
from multiprocessing.queues import (
    JoinableQueue as _JoinableQueue,
)
from multiprocessing.queues import (
    Queue as _Queue,
)
from multiprocessing.queues import (
    SimpleQueue as _SimpleQueue,
)
from multiprocessing.sharedctypes import SynchronizedBase
from multiprocessing.synchronize import (
    Barrier as _Barrier,
)
from multiprocessing.synchronize import (
    BoundedSemaphore as _BoundedSemaphore,
)
from multiprocessing.synchronize import (
    Condition as _Condition,
)
from multiprocessing.synchronize import (
    Event as _Event,
)
from multiprocessing.synchronize import (
    Lock as _Lock,
)
from multiprocessing.synchronize import (
    RLock as _RLock,
)
from multiprocessing.synchronize import (
    Semaphore as _Semaphore,
)
from typing import Any

from billiard import process
from billiard.connection import Connection
from billiard.exceptions import (
    SoftTimeLimitExceeded,
    TimeLimitExceeded,
    WorkerLostError,
)
from billiard.pool import Pool as _Pool
from billiard.process import BaseProcess
from typing_extensions import override

__all__: list[str] = []

W_NO_EXECV: str

class BaseContext:
    Process: type[BaseProcess]
    AuthenticationError: type[AuthenticationError]
    BufferTooShort: type[BufferTooShort]
    ProcessError: type[ProcessError]
    TimeoutError: type[TimeoutError]
    SoftTimeLimitExceeded: type[SoftTimeLimitExceeded]
    TimeLimitExceeded: type[TimeLimitExceeded]
    WorkerLostError: type[WorkerLostError]

    def Array(
        self,
        typecode_or_type: str | type[_CData],
        size_or_initializer: int | Iterable[Any],
        *args: Any,
        **kwargs: Any,
    ) -> _CTypesArray[Any]: ...
    def Barrier(
        self,
        parties: int,
        action: Callable[[], Any] | None = ...,
        timeout: float | None = ...,
    ) -> _Barrier: ...
    def BoundedSemaphore(self, value: int = ...) -> _BoundedSemaphore: ...
    def Condition(self, lock: _Lock | _RLock | None = ...) -> _Condition: ...
    def Event(self) -> _Event: ...
    def JoinableQueue(self, maxsize: int = ...) -> _JoinableQueue[Any]: ...
    def Lock(self) -> _Lock: ...
    def Manager(self) -> SyncManager: ...
    def Pipe(
        self, duplex: bool = ..., rnonblock: bool = ..., wnonblock: bool = ...
    ) -> tuple[Connection, Connection]: ...
    def Pool(
        self,
        processes: int | None = ...,
        initializer: Callable[..., Any] | None = ...,
        initargs: tuple[Any, ...] = ...,
        maxtasksperchild: int | None = ...,
        timeout: float | None = ...,
        soft_timeout: float | None = ...,
        lost_worker_timeout: float | None = ...,
        max_restarts: int | None = ...,
        max_restart_freq: int = ...,
        on_process_up: Callable[..., Any] | None = ...,
        on_process_down: Callable[..., Any] | None = ...,
        on_timeout_set: Callable[..., Any] | None = ...,
        on_timeout_cancel: Callable[..., Any] | None = ...,
        threads: bool = ...,
        semaphore: Any | None = ...,
        putlocks: bool = ...,
        allow_restart: bool = ...,
    ) -> _Pool: ...
    def Queue(self, maxsize: int = ...) -> _Queue[Any]: ...
    def RLock(self) -> _RLock: ...
    def RawArray(
        self,
        typecode_or_type: str | type[_CData],
        size_or_initializer: int | Iterable[Any],
    ) -> _CTypesArray[Any]: ...
    def RawValue(self, typecode_or_type: str | type[_CData], *args: Any) -> Any: ...
    def Semaphore(self, value: int = ...) -> _Semaphore: ...
    def SimpleQueue(self) -> _SimpleQueue[Any]: ...
    def Value(
        self, typecode_or_type: str | type[_CData], *args: Any, **kwargs: Any
    ) -> SynchronizedBase[Any]: ...
    @staticmethod
    def active_children(_cleanup: Callable[[], None] = ...) -> list[BaseProcess]: ...
    def allow_connection_pickling(self) -> None: ...
    def cpu_count(self) -> int: ...
    @staticmethod
    def current_process() -> BaseProcess: ...
    def forking_enable(self, value: bool) -> None: ...
    def forking_is_enabled(self) -> bool: ...
    def freeze_support(self) -> None: ...
    def get_context(self, method: str | None = ...) -> BaseContext: ...
    def get_logger(self) -> Logger: ...
    def get_start_method(self, allow_none: bool = ...) -> str | None: ...
    def log_to_stderr(self, level: int | None = ...) -> Logger: ...
    def set_executable(self, executable: str) -> None: ...
    def set_forkserver_preload(self, module_names: list[str]) -> None: ...
    def set_start_method(self, method: str | None = ...) -> None: ...

class Process(process.BaseProcess): ...

class DefaultContext(BaseContext):
    __all__: list[str]
    def __init__(self, context: BaseContext) -> None: ...
    def get_all_start_methods(self) -> list[str]: ...
    @override
    def set_start_method(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, method: str, force: bool = ...
    ) -> None: ...

class ForkProcess(process.BaseProcess): ...
class SpawnProcess(process.BaseProcess): ...
class ForkServerProcess(process.BaseProcess): ...

class ForkContext(BaseContext):
    Process: type[ForkProcess]  # pyright: ignore[reportIncompatibleVariableOverride]

class SpawnContext(BaseContext):
    Process: type[SpawnProcess]  # pyright: ignore[reportIncompatibleVariableOverride]

class ForkServerContext(BaseContext):
    Process: type[ForkServerProcess]  # pyright: ignore[reportIncompatibleVariableOverride]



from collections.abc import Callable, Mapping
from io import BytesIO
from types import TracebackType
from typing import Any

from typing_extensions import Self

__all__ = ("Headers", "Request", "Response")

class Headers(dict[str, str]):
    complete: bool
    _prev_key: str | None

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class Request:
    url: str
    method: str
    headers: Headers
    body: bytes | str | None
    connect_timeout: float
    request_timeout: float
    on_ready: Any  # promise
    on_timeout: Any | None
    on_stream: Any | None
    on_prepare: Any | None
    on_header: Any | None
    auth_username: str | None
    auth_password: str | None
    auth_mode: str | None
    user_agent: str | None
    network_interface: str | None
    use_gzip: bool
    validate_cert: bool
    ca_certs: str | None
    client_key: str | None
    client_cert: str | None
    proxy_host: str | None
    proxy_port: int | None
    proxy_username: str | None
    proxy_password: str | None
    follow_redirects: bool
    max_redirects: int

    def __init__(
        self,
        url: str,
        method: str = ...,
        on_ready: Callable[[Response], None] | None = ...,
        on_timeout: Callable[[], None] | None = ...,
        on_stream: Callable[[bytes], None] | None = ...,
        on_prepare: Callable[[Request], None] | None = ...,
        on_header: Callable[[Headers], None] | None = ...,
        headers: Mapping[str, str] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def then(
        self,
        callback: Callable[[Response], None],
        errback: Callable[[Exception], None] | None = ...,
    ) -> None: ...

class Response:
    request: Request
    code: int
    headers: Headers
    buffer: BytesIO | None
    effective_url: str
    error: Exception | None
    status: str
    _body: bytes | None

    def __init__(
        self,
        request: Request,
        code: int,
        headers: Headers | None = ...,
        buffer: BytesIO | None = ...,
        effective_url: str | None = ...,
        error: Exception | None = ...,
        status: str | None = ...,
    ) -> None: ...
    def raise_for_error(self) -> None: ...
    @property
    def body(self) -> bytes | None: ...
    @property
    def status_code(self) -> int: ...
    @property
    def content(self) -> bytes | None: ...

_Headers = Headers
_Request = Request
_Response = Response

class BaseClient:
    Headers: type[_Headers]
    Request: type[_Request]
    Response: type[_Response]
    hub: Any
    _header_parser: Any

    def __init__(self, hub: Any, **kwargs: Any) -> None: ...
    def perform(self, request: _Request | str, **kwargs: Any) -> None: ...
    def add_request(self, request: _Request) -> _Request: ...
    def close(self) -> None: ...
    def on_header(self, headers: _Headers, line: bytes | str) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...



from email.mime.message import MIMEMessage
from typing import Any

from kombu.asynchronous.http.base import Request as _Request
from vine import promise

__all__ = ("AsyncConnection", "AsyncHTTPSConnection")

def message_from_headers(hdr: list[tuple[str, str]]) -> Any: ...

class AsyncHTTPResponse:
    response: Any
    _msg: MIMEMessage | None
    version: int

    def __init__(self, response: Any) -> None: ...
    def read(self, *args: Any, **kwargs: Any) -> bytes: ...
    def getheader(self, name: str, default: str | None = ...) -> str | None: ...
    def getheaders(self) -> list[tuple[str, str]]: ...
    @property
    def msg(self) -> MIMEMessage: ...
    @property
    def status(self) -> int: ...
    @property
    def reason(self) -> str: ...

_AsyncHTTPResponse = AsyncHTTPResponse

class AsyncHTTPSConnection:
    Request: type[_Request]
    Response: type[_AsyncHTTPResponse]

    method: str
    path: str
    body: bytes | None
    default_ports: dict[str, int]
    headers: list[tuple[str, str]]
    timeout: float
    strict: Any
    http_client: Any

    def __init__(
        self, strict: Any = ..., timeout: float = ..., http_client: Any = ...
    ) -> None: ...
    def request(
        self, method: str, path: str, body: Any = ..., headers: Any = ...
    ) -> None: ...
    def getrequest(self) -> _Request: ...
    def getresponse(self, callback: Any = ...) -> Any: ...
    def set_debuglevel(self, level: int) -> None: ...
    def connect(self) -> None: ...
    def close(self) -> None: ...
    def putrequest(self, method: str, path: str) -> None: ...
    def putheader(self, header: str, value: str) -> None: ...
    def endheaders(self) -> None: ...
    def send(self, data: bytes) -> None: ...

class AsyncConnection:
    sqs_connection: Any
    _httpclient: Any

    def __init__(
        self, sqs_connection: Any, http_client: Any = ..., **kwargs: Any
    ) -> None: ...
    def get_http_connection(self) -> AsyncHTTPSConnection: ...

class AsyncAWSQueryConnection(AsyncConnection):
    STATUS_CODE_OK: int
    STATUS_CODE_REQUEST_TIMEOUT: int
    STATUS_CODE_NETWORK_CONNECT_TIMEOUT_ERROR: int
    STATUS_CODE_INTERNAL_ERROR: int
    STATUS_CODE_BAD_GATEWAY: int
    STATUS_CODE_SERVICE_UNAVAILABLE_ERROR: int
    STATUS_CODE_GATEWAY_TIMEOUT: int
    STATUS_CODES_SERVER_ERRORS: tuple[int, ...]
    STATUS_CODES_TIMEOUT: tuple[int, ...]

    def __init__(
        self,
        sqs_connection: Any,
        http_client: Any = ...,
        http_client_params: dict[str, Any] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def make_request(
        self,
        operation: str,
        params_: dict[str, Any],
        path: str,
        verb: str,
        callback: Any = ...,
        protocol_params: dict[str, Any] | None = ...,
    ) -> promise: ...
    def get_list(
        self,
        operation: str,
        params: dict[str, Any],
        markers: Any,
        path: str = ...,
        parent: Any = ...,
        verb: str = ...,
        callback: Any = ...,
        protocol_params: dict[str, Any] | None = ...,
    ) -> promise: ...
    def get_object(
        self,
        operation: str,
        params: dict[str, Any],
        path: str = ...,
        parent: Any = ...,
        verb: str = ...,
        callback: Any = ...,
        protocol_params: dict[str, Any] | None = ...,
    ) -> promise: ...
    def get_status(
        self,
        operation: str,
        params: dict[str, Any],
        path: str = ...,
        parent: Any = ...,
        verb: str = ...,
        callback: Any = ...,
        protocol_params: dict[str, Any] | None = ...,
    ) -> promise: ...



from collections.abc import Callable
from typing import Any

__all__ = (
    "UnpickleableExceptionWrapper",
    "create_exception_cls",
    "find_pickleable_exception",
    "get_pickleable_etype",
    "get_pickleable_exception",
    "get_pickled_exception",
    "strtobool",
    "subclass_exception",
)

STRTOBOOL_DEFAULT_TABLE: dict[str, bool]

class UnpickleableExceptionWrapper(Exception):
    exc_module: str | None
    exc_cls_name: str | None
    exc_args: tuple[Any, ...] | None

    def __init__(
        self,
        exc_module: str,
        exc_cls_name: str,
        exc_args: tuple[Any, ...],
        text: str | None = None,
    ) -> None: ...
    @classmethod
    def from_exception(cls, exc: Exception) -> UnpickleableExceptionWrapper: ...
    def restore(self) -> Exception: ...

def b64encode(s: bytes) -> bytes: ...
def b64decode(s: bytes) -> bytes: ...
def base64encode(s: bytes, altchars: bytes | None = None) -> bytes: ...
def base64decode(
    s: bytes, altchars: bytes | None = None, validate: bool = False
) -> bytes: ...
def bytes_to_str(s: bytes | str) -> str: ...
def str_to_bytes(s: str | bytes) -> bytes: ...
def strtobool(
    term: str | bool | int | None, table: dict[str, bool] | None = None
) -> bool: ...
def get_pickleable_exception(exc: Exception) -> Exception: ...
def get_pickleable_etype(
    cls: type[Exception], loads: Any = ..., dumps: Any = ...
) -> type[Exception]: ...
def get_pickled_exception(exc: Exception) -> Exception: ...
def jsonify(
    obj: Any,
    builtin_types: tuple[type, ...] = ...,
    key: Any = None,
    keyfilter: Callable[[str], bool] | None = None,
    unknown_type_filter: Callable[[Any], Any] | None = None,
) -> Any: ...
def raise_with_context(exc: Exception) -> None: ...
def find_pickleable_exception(
    exc: Exception,
    loads: Any = ...,
    dumps: Any = ...,
) -> Exception | None: ...
def ensure_serializable(items: list[Any], encoder: Any) -> list[Any]: ...
def create_exception_cls(
    name: str, module: str, parent: type | None = None
) -> type[Exception]: ...
def subclass_exception(
    name: str, parent: type[Exception], module: str
) -> type[Exception]: ...
def itermro(cls: type, stop: type) -> Any: ...
def safe_repr(o: Any, errors: str = "replace") -> str: ...



from collections.abc import Sequence

__all__ = [
    "connect_to_new_process",
    "ensure_running",
    "get_inherited_fds",
    "set_forkserver_preload",
]

def connect_to_new_process(fds: Sequence[int]) -> tuple[int, int]: ...
def ensure_running() -> None: ...
def get_inherited_fds() -> list[int] | None: ...
def set_forkserver_preload(modules_names: list[str]) -> None: ...



import re
from collections.abc import Callable, Generator, Mapping
from contextlib import contextmanager
from socket import socket
from ssl import SSLError as SSLError
from typing import Any

from amqp.exceptions import UnexpectedFrame as UnexpectedFrame

AMQP_PORT: int
AMQP_PROTOCOL_HEADER: bytes
DEFAULT_SOCKET_SETTINGS: dict[str, Any]
EMPTY_BUFFER: bytes
IPV6_LITERAL: re.Pattern[str]
KNOWN_TCP_OPTS: set[str]
SIGNED_INT_MAX: int
SOL_TCP: int

class TCPTransport:
    host: str
    port: int
    connect_timeout: float | None
    read_timeout: float | None
    write_timeout: float | None
    socket_settings: Mapping[str, Any] | None
    sock: socket | None
    connection: Any
    raise_on_initial_eintr: bool

    def __init__(
        self,
        host: str,
        connect_timeout: float | None = ...,
        read_timeout: float | None = ...,
        write_timeout: float | None = ...,
        socket_settings: Mapping[str, Any] | None = ...,
        raise_on_initial_eintr: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    def connect(self) -> None: ...
    def close(self) -> None: ...
    def read_frame(
        self, unpack: Callable[[str, bytes], tuple[Any, ...]] = ...
    ) -> tuple[int, int, bytes]: ...
    def write(self, s: bytes) -> None: ...
    @contextmanager
    def having_timeout(self, timeout: float | None) -> Generator[None, None, None]: ...

class SSLTransport(TCPTransport):
    sslopts: dict[str, Any] | bool | None

    def __init__(
        self,
        host: str,
        connect_timeout: float | None = ...,
        ssl: bool | dict[str, Any] | None = ...,
        **kwargs: Any,
    ) -> None: ...

def Transport(
    host: str,
    connect_timeout: float | None = ...,
    ssl: bool | dict[str, Any] = ...,
    **kwargs: Any,
) -> TCPTransport | SSLTransport: ...
def to_host_port(host: str, default: int = ...) -> tuple[str, int]: ...
def set_cloexec(fd: int, cloexec: bool) -> None: ...



import io
import numbers
import os
from collections.abc import Iterator, Sequence
from typing import SupportsInt, TypeAlias, TypeVar

from _typeshed import FileDescriptorLike, ReadableBuffer, StrOrBytesPath

FILENO_ERRORS: tuple[
    type[AttributeError], type[ValueError], type[io.UnsupportedOperation]
]
__write__ = os.write

def send_offset(fd: int, buf: ReadableBuffer, offset: int) -> int: ...

fsencode = os.fsencode
fsdecode = os.fsdecode

# Note: MaybeFileNo is only used internally, not exported at runtime
# but we need it for type annotations
_MaybeFileNo: TypeAlias = numbers.Integral | io.IOBase

def maybe_fileno(f: _MaybeFileNo) -> numbers.Integral: ...
def get_fdmax(default: int | None = ...) -> int | None: ...

_T = TypeVar("_T")

def uniq(it: Sequence[_T]) -> Iterator[_T]: ...

closerange = os.closerange

def close_open_fds(keep: Sequence[_MaybeFileNo] | None = ...) -> None: ...
def get_errno(exc: Exception | None) -> int: ...
def spawnv_passfds(
    path: StrOrBytesPath, args: os._ExecVArgs, passfds: Sequence[_MaybeFileNo]
) -> int: ...
def isblocking(handle: FileDescriptorLike) -> bool: ...
def setblocking(handle: FileDescriptorLike, blocking: bool) -> None: ...

E_PSUTIL_MISSING: str
E_RESOURCE_MISSING: str

def maxrss_to_kb(v: SupportsInt) -> int: ...
def mem_rss() -> int: ...



import json
from collections.abc import Callable
from typing import Any, TypeAlias, TypeVar

# These TypeVars are exported by the runtime module, so they must be public
T = TypeVar("T")  # noqa: PYI001
EncodedT = TypeVar("EncodedT")  # noqa: PYI001

textual_types: tuple[type, ...]

class JSONEncoder(json.JSONEncoder): ...

def dumps(
    s: Any,
    _dumps: Callable[..., str] = ...,
    cls: type[json.JSONEncoder] = ...,
    default_kwargs: dict[str, Any] | None = ...,
    **kwargs: Any,
) -> str: ...
def object_hook(o: dict[Any, Any]) -> Any: ...
def loads(
    s: str | bytes | bytearray | memoryview,
    _loads: Callable[[str], Any] = ...,
    decode_bytes: bool = ...,
    object_hook: Callable[[dict[Any, Any]], Any] | None = ...,
) -> Any: ...

EncoderT: TypeAlias = Callable[[Any], Any]
DecoderT: TypeAlias = Callable[[Any], Any]

_T = TypeVar("_T")
_EncodedT = TypeVar("_EncodedT")

def register_type(
    t: type[_T],
    marker: str | None,
    encoder: Callable[[_T], _EncodedT],
    decoder: Callable[[_EncodedT], _T] = ...,
) -> None: ...



TRANSPORT_ALIASES: dict[str, str]

def get_transport_cls(transport: str | None = ...) -> str | None: ...
def resolve_transport(transport: str | None = ...) -> str | None: ...
def supports_librabbitmq() -> bool | None: ...



import boto3 as boto3
from botocore import exceptions as exceptions
from botocore.awsrequest import AWSRequest as AWSRequest
from botocore.httpsession import get_cert_path as get_cert_path
from botocore.response import get_response as get_response

__all__ = ("AWSRequest", "exceptions", "get_cert_path", "get_response")



from decimal import Decimal as Decimal
from typing import Any

from amqp.exceptions import FrameSyntaxError as FrameSyntaxError
from amqp.spec import Basic as Basic

ILLEGAL_TABLE_TYPE: str
ILLEGAL_TABLE_TYPE_WITH_KEY: str
ILLEGAL_TABLE_TYPE_WITH_VALUE: str
PROPERTY_CLASSES: dict[int, type[Any]]

class GenericContent:
    CLASS_ID: int | None
    PROPERTIES: list[tuple[str, str]]

    body_size: int
    body_received: int
    properties: dict[str, Any]
    ready: bool
    frame_method: tuple[int, int] | None
    frame_args: bytes | None

    def __init__(
        self,
        frame_method: tuple[int, int] | None = ...,
        frame_args: bytes | None = ...,
        **props: Any,
    ) -> None: ...
    def inbound_header(self, buf: bytes, offset: int = ...) -> None: ...
    def inbound_body(self, buf: bytes) -> None: ...

def dumps(format: str, values: tuple[Any, ...]) -> bytes: ...
def loads(format: str, buf: bytes, offset: int) -> tuple[Any, ...]: ...
def decode_properties_basic(buf: bytes, offset: int) -> tuple[dict[str, Any], int]: ...
def pstr_t(s: str) -> bytes: ...
def str_to_bytes(s: str) -> bytes: ...



from typing import Any

import kombu
from celery.backends.asynchronous import BaseResultConsumer
from celery.backends.base import Backend
from kombu.utils.objects import cached_property
from typing_extensions import override

__all__ = ("BacklogLimitExceeded", "RPCBackend")

class BacklogLimitExceeded(Exception): ...

class RPCBackend(Backend):
    BacklogLimitExceeded: type[BacklogLimitExceeded]
    Consumer: type[kombu.Consumer]
    Exchange: type[kombu.Exchange]
    Producer: type[kombu.Producer]
    Queue: type[kombu.Queue]
    ResultConsumer: type[BaseResultConsumer]

    oid: cached_property[str]

    @property
    def binding(self) -> kombu.Queue: ...
    def __init__(
        self,
        app: Any,
        connection: Any | None = None,
        exchange: str | None = None,
        exchange_type: str | None = None,
        persistent: bool | None = None,
        serializer: str | None = None,
        auto_delete: bool = True,
        **kwargs: Any,
    ) -> None: ...
    def destination_for(self, task_id: str, request: Any) -> tuple[str, str]: ...
    def on_reply_declare(self, task_id: str) -> Any: ...
    def on_result_fulfilled(self, result: Any) -> None: ...
    def on_out_of_band_result(self, task_id: str, message: Any) -> None: ...
    @override
    def get_task_meta(  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
        self, task_id: str, backlog_limit: int = 1000
    ) -> dict[str, Any]: ...
    def poll(
        self, task_id: str, backlog_limit: int = 1000
    ) -> dict[str, Any] | None: ...
    @override
    def reload_group_result(self, task_id: str) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    def revive(self, channel: Any) -> None: ...



from typing import Any

__all__ = ("TaskRegistry",)

from celery.app.task import Task
from celery.exceptions import NotRegistered

class TaskRegistry(dict[str, Task[Any, Any]]):
    NotRegistered: type[NotRegistered]

    def __missing__(self, key: str) -> Task[Any, Any]: ...
    def filter_types(self, type: str) -> dict[str, Task[Any, Any]]: ...
    def periodic(self) -> dict[str, Task[Any, Any]]: ...
    def register(self, task: Task[Any, Any]) -> Task[Any, Any]: ...
    def regular(self) -> dict[str, Task[Any, Any]]: ...
    def unregister(self, name: str) -> None: ...



from collections.abc import Callable, Iterable

__all__ = (
    "compress",
    "decompress",
    "encoders",
    "get_decoder",
    "get_encoder",
    "register",
)

def register(
    encoder: Callable[[bytes], bytes],
    decoder: Callable[[bytes], bytes],
    content_type: str,
    aliases: Iterable[str] | None = ...,
) -> None: ...
def encoders() -> list[str]: ...
def get_encoder(t: str) -> Callable[[bytes], bytes]: ...
def get_decoder(t: str) -> Callable[[bytes], bytes]: ...
def compress(body: bytes | str, content_type: str) -> tuple[bytes, str]: ...
def decompress(body: bytes, content_type: str) -> bytes: ...



from collections.abc import Callable, Mapping, Sequence
from types import TracebackType
from typing import Any
from unittest.mock import Mock

from celery import Celery
from celery.canvas import Signature
from typing_extensions import Self

def TaskMessage(
    name: str,
    id: str | None = None,
    args: Sequence[Any] = (),
    kwargs: Mapping[Any, Any] | None = None,
    callbacks: Sequence[Signature[Any]] | None = None,
    errbacks: Sequence[Signature[Any]] | None = None,
    chain: Sequence[Signature[Any]] | None = None,
    shadow: str | None = None,
    utc: bool | None = None,
    **options: Any,
) -> Any: ...
def TaskMessage1(
    name: str,
    id: str | None = None,
    args: Sequence[Any] = (),
    kwargs: Mapping[Any, Any] | None = None,
    callbacks: Sequence[Signature[Any]] | None = None,
    errbacks: Sequence[Signature[Any]] | None = None,
    chain: Sequence[Signature[Any]] | None = None,
    **options: Any,
) -> Any: ...
def task_message_from_sig(
    app: Celery,
    sig: Signature[Any],
    utc: bool = True,
    TaskMessage: Callable[..., Any] = ...,
) -> Any: ...

class _ContextMock(Mock):
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

def ContextMock(*args: Any, **kwargs: Any) -> _ContextMock: ...



from typing import Any

from celery.backends.base import KeyValueStoreBackend

__all__ = ("GCSBackend",)

class GCSBackend(KeyValueStoreBackend):
    @property
    def firestore_client(self) -> Any: ...
    def __init__(self, **kwargs: Any) -> None: ...



from typing import Any

from celery.backends.base import KeyValueStoreBackend

__all__ = ("CouchbaseBackend",)

class CouchbaseBackend(KeyValueStoreBackend):
    bucket: str
    host: str
    port: int
    username: str | None
    password: str | None
    quiet: bool
    timeout: float
    key_t: type[str]  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleMethodOverride]

    @property
    def connection(self) -> Any: ...
    def __init__(self, url: str | None = None, *args: Any, **kwargs: Any) -> None: ...



from typing import Any

from kombu import Connection
from kombu.transport.virtual import Channel

def establish_connection(
    hostname: str | None = ...,
    userid: str | None = ...,
    password: str | None = ...,
    virtual_host: str | None = ...,
    port: int | None = ...,
    ssl: bool | dict[str, Any] | None = ...,
    connect_timeout: float | None = ...,
    transport: str | None = ...,
    transport_options: dict[str, Any] | None = ...,
    heartbeat: float | None = ...,
    login_method: str | None = ...,
    failover_strategy: str | None = ...,
    **kwargs: Any,
) -> Connection: ...
def get_consumer_set(
    channel: Channel,
    queues: Any | None = ...,
    accept: list[str] | None = ...,
    **kw: Any,
) -> Any: ...
def TaskConsumer(
    channel: Channel,
    queues: Any | None = ...,
    accept: list[str] | None = ...,
    **kw: Any,
) -> Any: ...

__all__ = ["TaskConsumer", "establish_connection", "get_consumer_set"]



from collections.abc import Generator, Iterable, Iterator
from types import TracebackType
from typing import Any

from kombu.connection import Connection
from kombu.entity import Exchange, Queue
from kombu.message import Message
from kombu.messaging import Consumer as _Consumer
from kombu.messaging import Producer
from kombu.transport.base import StdChannel
from typing_extensions import Self, override

__all__ = ("Consumer", "Publisher")

def entry_to_queue(queue: str, **options: Any) -> Queue: ...

class Publisher(Producer):
    exchange: Exchange | str  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleVariableOverride]
    exchange_type: str
    routing_key: str
    durable: bool
    auto_delete: bool
    _closed: bool

    def __init__(
        self,
        connection: Connection | StdChannel,
        exchange: str | Exchange | None = ...,
        routing_key: str | None = ...,
        exchange_type: str | None = ...,
        durable: bool | None = ...,
        auto_delete: bool | None = ...,
        channel: StdChannel | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def send(self, *args: Any, **kwargs: Any) -> Any: ...
    @override
    def close(self) -> None: ...
    @override
    def __enter__(self) -> Self: ...
    @override
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    @property
    def backend(self) -> StdChannel: ...

class Consumer(_Consumer):
    queue: str
    exchange: str
    routing_key: str
    exchange_type: str
    durable: bool
    exclusive: bool
    auto_delete: bool
    backend: StdChannel
    _closed: bool

    def __init__(
        self,
        connection: Connection,
        queue: str | None = ...,
        exchange: str | None = ...,
        routing_key: str | None = ...,
        exchange_type: str | None = ...,
        durable: bool | None = ...,
        exclusive: bool | None = ...,
        auto_delete: bool | None = ...,
        **kwargs: Any,
    ) -> None: ...
    @override
    def revive(self, channel: StdChannel) -> None: ...
    @override
    def close(self) -> None: ...
    @override
    def __enter__(self) -> Self: ...
    @override
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    def __iter__(self) -> Iterator[Message | None]: ...
    def fetch(
        self, no_ack: bool | None = ..., enable_callbacks: bool = ...
    ) -> Message | None: ...
    def process_next(self) -> None: ...
    def discard_all(self, filterfunc: Any = ...) -> int: ...
    def iterconsume(
        self, limit: int | None = ..., no_ack: bool | None = ...
    ) -> Generator[Any, None, None]: ...
    def wait(self, limit: int | None = ...) -> list[Any]: ...
    def iterqueue(
        self, limit: int | None = ..., infinite: bool = ...
    ) -> Generator[Message | None, None, None]: ...

class ConsumerSet(_Consumer):
    backend: StdChannel
    _provided_channel: bool

    def __init__(
        self,
        connection: Connection,
        from_dict: dict[str, dict[str, Any]] | None = ...,
        consumers: Iterable[_Consumer] | None = ...,
        channel: StdChannel | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def iterconsume(
        self, limit: int | None = ..., no_ack: bool = ...
    ) -> Generator[Any, None, None]: ...
    def discard_all(self) -> int: ...
    def add_consumer_from_dict(self, queue: str, **options: Any) -> Queue: ...
    def add_consumer(self, consumer: _Consumer) -> None: ...
    @override
    def revive(self, channel: StdChannel) -> None: ...
    @override
    def close(self) -> None: ...



from collections.abc import Callable, Iterator, Mapping
from contextlib import contextmanager
from datetime import datetime
from types import TracebackType
from typing import (
    Any,
    Generic,
    Literal,
    TypeAlias,
    TypeVar,
)

__all__ = (
    "AsyncResult",
    "EagerResult",
    "GroupResult",
    "ResultBase",
    "ResultSet",
    "result_from_tuple",
)

import kombu
from celery.app.base import Celery
from celery.backends.base import Backend
from celery.exceptions import TimeoutError as CeleryTimeoutError
from celery.utils.graph import DependencyGraph, GraphFormatter
from typing_extensions import override
from vine import promise

@contextmanager
def denied_join_result() -> Iterator[None]: ...
@contextmanager
def allow_join_result() -> Iterator[None]: ...

class ResultBase:
    parent: ResultBase | None

_State: TypeAlias = Literal["PENDING", "STARTED", "RETRY", "FAILURE", "SUCCESS"]

_R_co = TypeVar("_R_co", covariant=True)

class AsyncResult(ResultBase, Generic[_R_co]):
    TimeoutError: type[CeleryTimeoutError]
    app: Celery | None
    id: str | None
    backend: Backend | None
    parent: ResultBase | None
    on_ready: promise
    def __init__(
        self,
        id: str,
        backend: Backend | None = ...,
        task_name: str | None = ...,
        app: Celery | None = ...,
        parent: ResultBase | None = ...,
    ) -> None: ...
    def __copy__(self) -> AsyncResult[_R_co]: ...
    def __del__(self) -> None: ...
    def __reduce_args__(self) -> tuple[str, Backend]: ...
    @property
    def ignored(self) -> bool: ...
    @ignored.setter
    def ignored(self, value: bool) -> None: ...
    def then(
        self,
        callback: Callable[..., Any],
        on_error: Callable[..., Any] | None = ...,
        weak: bool = ...,
    ) -> promise: ...
    def as_tuple(
        self,
    ) -> tuple[int, tuple[int, Any | None, None] | None, None]: ...
    def forget(self) -> None: ...
    def revoke(
        self,
        connection: kombu.Connection | None = ...,
        terminate: bool = ...,
        signal: str | None = ...,
        wait: bool = ...,
        timeout: float | None = ...,
    ) -> None: ...
    def get(
        self,
        timeout: float | None = ...,
        propagate: bool = ...,
        interval: float = ...,
        no_ack: bool = ...,
        follow_parents: bool = ...,
        callback: Callable[..., Any] | None = ...,
        on_message: Callable[..., Any] | None = ...,
        on_interval: Callable[..., Any] | None = ...,
        disable_sync_subtasks: bool = ...,
        EXCEPTION_STATES: frozenset[str] = ...,
        PROPAGATE_STATES: frozenset[str] = ...,
    ) -> _R_co: ...
    def collect(
        self, intermediate: bool = ..., **kwargs: Any
    ) -> Iterator[tuple[AsyncResult[Any], object]]: ...
    def get_leaf(self) -> object: ...
    def iterdeps(
        self, intermediate: bool = ...
    ) -> Iterator[tuple[AsyncResult[Any] | None, AsyncResult[Any]]]: ...
    def ready(self) -> bool: ...
    def successful(self) -> bool: ...
    def failed(self) -> bool: ...
    def throw(self, *args: Any, **kwargs: Any) -> None: ...
    def maybe_throw(
        self, propagate: bool = ..., callback: Callable[..., Any] | None = ...
    ) -> object: ...
    def maybe_reraise(
        self, propagate: bool = ..., callback: Callable[..., Any] | None = ...
    ) -> None: ...
    def as_list(self) -> list[AsyncResult[Any]]: ...
    def wait(
        self,
        timeout: float | None = ...,
        propagate: bool = ...,
        interval: float = ...,
        no_ack: bool = ...,
        follow_parents: bool = ...,
        callback: Callable[..., Any] | None = ...,
        on_message: Callable[..., Any] | None = ...,
        on_interval: Callable[..., Any] | None = ...,
        disable_sync_subtasks: bool = ...,
        EXCEPTION_STATES: frozenset[str] = ...,
        PROPAGATE_STATES: frozenset[str] = ...,
    ) -> _R_co: ...
    def revoke_by_stamped_headers(
        self,
        headers: list[str],
        connection: kombu.Connection | None = ...,
        terminate: bool = ...,
        signal: str | None = ...,
        wait: bool = ...,
        timeout: float | None = ...,
    ) -> None: ...
    def build_graph(
        self, intermediate: bool = ..., formatter: GraphFormatter | None = ...
    ) -> DependencyGraph: ...
    @property
    def graph(self) -> DependencyGraph: ...
    @property
    def supports_native_join(self) -> bool: ...
    @property
    def children(
        self,
    ) -> list[tuple[int, tuple[int, Any | None, None] | None, None]] | None: ...
    @property
    def result(self) -> _R_co | BaseException: ...
    @property
    def info(self) -> Any: ...
    @property
    def traceback(self) -> TracebackType | None: ...
    @property
    def state(self) -> _State: ...
    @property
    def status(
        self,
    ) -> _State: ...
    @property
    def task_id(self) -> str: ...
    @task_id.setter
    def task_id(self, id: str) -> None: ...
    @property
    def name(self) -> str | None: ...
    @property
    def args(self) -> tuple[Any, ...] | None: ...
    @property
    def kwargs(self) -> Mapping[str, Any] | None: ...
    @property
    def worker(self) -> str | None: ...
    @property
    def date_done(self) -> datetime | None: ...
    @property
    def retries(self) -> int | None: ...
    @property
    def queue(self) -> str | None: ...

class EagerResult(AsyncResult[_R_co]):
    def __init__(
        self,
        id: str,
        ret_value: _R_co,
        state: str,
        traceback: str | None = ...,
        name: str | None = ...,
    ) -> None: ...
    @override
    def __copy__(self) -> EagerResult[_R_co]: ...
    @override
    def __reduce_args__(self) -> tuple[str, _R_co, str, str | None]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def get(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        timeout: float | None = ...,
        propagate: bool = ...,
        disable_sync_subtasks: bool = ...,
        **kwargs: Any,
    ) -> _R_co: ...
    @override
    def wait(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        timeout: float | None = ...,
        propagate: bool = ...,
        disable_sync_subtasks: bool = ...,
        **kwargs: Any,
    ) -> _R_co: ...

class ResultSet(ResultBase):
    results: list[AsyncResult[Any]] | None
    app: Celery
    def __init__(
        self,
        results: list[AsyncResult[Any]] | None,
        app: Celery | None = ...,
        ready_barrier: Any | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def __getitem__(self, index: int) -> AsyncResult[Any]: ...
    def __iter__(self) -> Iterator[AsyncResult[Any]]: ...
    def __len__(self) -> int: ...
    def add(self, result: AsyncResult[Any]) -> None: ...
    @property
    def backend(self) -> Backend: ...
    def clear(self) -> None: ...
    def completed_count(self) -> int: ...
    def discard(self, result: AsyncResult[Any]) -> None: ...
    def failed(self) -> bool: ...
    def forget(self) -> None: ...
    def get(
        self,
        timeout: float | None = ...,
        propagate: bool = ...,
        interval: float = ...,
        callback: Callable[..., Any] | None = ...,
        no_ack: bool = ...,
        on_message: Callable[..., Any] | None = ...,
        disable_sync_subtasks: bool = ...,
        on_interval: Callable[..., Any] | None = ...,
    ) -> list[Any]: ...
    def iter_native(
        self,
        timeout: float | None = ...,
        interval: float = ...,
        no_ack: bool = ...,
        on_message: Callable[..., Any] | None = ...,
        on_interval: Callable[..., Any] | None = ...,
    ) -> Iterator[tuple[str, Any]]: ...
    def join(
        self,
        timeout: float | None = ...,
        propagate: bool = ...,
        interval: float = ...,
        callback: Callable[..., Any] | None = ...,
        no_ack: bool = ...,
        on_message: Callable[..., Any] | None = ...,
        disable_sync_subtasks: bool = ...,
        on_interval: Callable[..., Any] | None = ...,
    ) -> list[Any]: ...
    def join_native(
        self,
        timeout: float | None = ...,
        propagate: bool = ...,
        interval: float = ...,
        callback: Callable[..., Any] | None = ...,
        no_ack: bool = ...,
        on_message: Callable[..., Any] | None = ...,
        on_interval: Callable[..., Any] | None = ...,
        disable_sync_subtasks: bool = ...,
    ) -> list[Any]: ...
    def maybe_reraise(
        self, callback: Callable[..., Any] | None = ..., propagate: bool = ...
    ) -> None: ...
    def maybe_throw(
        self, callback: Callable[..., Any] | None = ..., propagate: bool = ...
    ) -> None: ...
    def ready(self) -> bool: ...
    def remove(self, result: AsyncResult[Any]) -> None: ...
    def revoke(
        self,
        connection: kombu.Connection | None = ...,
        terminate: bool = ...,
        signal: str | None = ...,
        wait: bool = ...,
        timeout: float | None = ...,
    ) -> None: ...
    def successful(self) -> bool: ...
    @property
    def supports_native_join(self) -> bool: ...
    def then(
        self,
        callback: Callable[..., Any],
        on_error: Callable[..., Any] | None = ...,
        weak: bool = ...,
    ) -> promise: ...
    def update(self, results: list[AsyncResult[Any]]) -> None: ...
    def waiting(self) -> bool: ...

def result_from_tuple(
    r: tuple[Any, ...], app: Celery | None = ...
) -> AsyncResult[Any]: ...

class GroupResult(ResultSet):
    id: str | None
    def __init__(
        self,
        id: str | None = ...,
        results: list[AsyncResult[Any]] | None = ...,
        parent: ResultBase | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def __bool__(self) -> bool: ...
    def __reduce_args__(self) -> tuple[str | None, list[AsyncResult[Any]] | None]: ...
    def as_tuple(self) -> tuple[str | None, list[tuple[Any, ...]] | None]: ...
    @classmethod
    def restore(
        cls,
        id: str,
        backend: Backend | None = ...,
        app: Celery | None = ...,
    ) -> GroupResult: ...
    def save(self, backend: Backend | None = ...) -> Any: ...
    def delete(self, backend: Backend | None = ...) -> None: ...
    @property
    def children(self) -> list[AsyncResult[Any]] | None: ...



from collections.abc import Callable, Iterator
from datetime import datetime
from decimal import Decimal
from typing import Any

from kombu.utils.objects import cached_property

__all__ = ("State", "Task", "Worker", "heartbeat_expires")

def heartbeat_expires(
    timestamp: float,
    freq: float = 60,
    expire_window: float = 200,
    Decimal: type[Decimal] = ...,
    float: type[float] = ...,
    isinstance: Callable[..., bool] = ...,
) -> float: ...

class Task:
    # Attributes based on runtime inspection
    uuid: str
    name: str | None
    state: str
    received: float | None
    sent: float | None
    started: float | None
    succeeded: float | None
    failed: float | None
    retried: float | None
    revoked: float | None
    rejected: float | None
    args: str | None
    kwargs: str | None
    eta: datetime | None
    expires: datetime | None
    retries: int | None
    result: Any
    exception: str | None
    traceback: str | None
    worker: Worker | None
    client: str | None
    clock: int | None
    timestamp: float | None
    root_id: str | None
    parent_id: str | None
    exchange: str | None
    routing_key: str | None
    runtime: float | None

    merge_rules: dict[str, Any]

    @property
    def id(self) -> str | None: ...
    @property
    def origin(self) -> str | None: ...
    root: cached_property[str | None]
    parent: cached_property[str | None]

    def __init__(
        self,
        uuid: str | None = None,
        cluster_state: State | None = None,
        children: list[str] | None = None,
        **kwargs: Any,
    ) -> None: ...
    def event(
        self,
        type_: str | None,
        timestamp: float | None = None,
        local_received: float | None = None,
        fields: dict[str, Any] | None = None,
        precedence: Callable[..., int] = ...,
        setattr: Callable[..., None] = ...,
        task_event_to_state: Callable[..., str | None] = ...,
        RETRY: str = "RETRY",
    ) -> str | None: ...
    def info(
        self,
        fields: list[str] | None = None,
        extra: list[str] | None = None,
    ) -> dict[str, Any]: ...
    def as_dict(self) -> dict[str, Any]: ...
    @property
    def ready(self) -> bool: ...

class Worker:
    hostname: str
    pid: int | None
    freq: float
    heartbeats: list[float]
    clock: int
    active: int | None
    processed: int | None
    loadavg: list[float] | None
    sw_ident: str | None
    sw_ver: str | None
    sw_sys: str | None
    expire_window: float
    heartbeat_max: int
    event: Any  # member_descriptor at runtime

    @property
    def id(self) -> str | None: ...
    def __init__(
        self,
        hostname: str | None = None,
        pid: int | None = None,
        freq: float = 60,
        heartbeats: list[float] | None = None,
        clock: int = 0,
        active: int | None = None,
        processed: int | None = None,
        loadavg: list[float] | None = None,
        sw_ident: str | None = None,
        sw_ver: str | None = None,
        sw_sys: str | None = None,
    ) -> None: ...
    def update(
        self,
        f: dict[str, Any],
        **kw: Any,
    ) -> None: ...
    @property
    def heartbeat_expires(self) -> float: ...
    @property
    def alive(self) -> bool: ...
    @property
    def status_string(self) -> str: ...

_Task = Task
_Worker = Worker

class State:
    event_count: int
    task_count: int
    heap_multiplier: int
    max_workers_in_memory: int
    max_tasks_in_memory: int

    Task: type[_Task]
    Worker: type[_Worker]

    def __init__(
        self,
        callback: Callable[..., Any] | None = None,
        workers: dict[str, _Worker] | None = None,
        tasks: dict[str, _Task] | None = None,
        taskheap: Any = None,
        max_workers_in_memory: int = 5000,
        max_tasks_in_memory: int = 10000,
        on_node_join: Callable[..., Any] | None = None,
        on_node_leave: Callable[..., Any] | None = None,
        tasks_by_type: Callable[..., Any] | None = None,
        tasks_by_worker: Callable[..., Any] | None = None,
    ) -> None: ...
    def freeze_while(
        self, fun: Callable[..., Any], *args: Any, **kwargs: Any
    ) -> Any: ...
    def clear_tasks(self, ready: bool = True) -> None: ...
    def clear(self, ready: bool = True) -> None: ...
    def get_or_create_worker(
        self, hostname: str, **kwargs: Any
    ) -> tuple[_Worker, bool]: ...
    def get_or_create_task(self, uuid: str) -> tuple[_Task, bool]: ...
    def worker_event(
        self, type_: str, fields: dict[str, Any]
    ) -> tuple[_Worker, bool] | tuple[None, bool]: ...
    def task_event(
        self, type_: str, fields: dict[str, Any]
    ) -> tuple[_Task, bool] | tuple[None, bool]: ...
    def event(self, event: dict[str, Any]) -> tuple[_Worker | _Task | None, bool]: ...
    def rebuild_taskheap(self, timetuple: Callable[..., Any] = ...) -> None: ...
    def itertasks(self, limit: int | None = None) -> Iterator[tuple[str, _Task]]: ...
    def tasks_by_time(
        self, limit: int | None = None, reverse: bool = True
    ) -> list[tuple[str, _Task]]: ...
    def tasks_by_timestamp(
        self, limit: int | None = None, reverse: bool = True
    ) -> list[tuple[str, _Task]]: ...
    def task_types(self) -> set[str]: ...
    def alive_workers(self) -> list[_Worker]: ...



import re
from datetime import datetime
from types import ModuleType
from typing import Any

from celery.app.base import Celery

__all__ = ("BaseLoader",)

class BaseLoader:
    app: Celery
    builtin_modules: frozenset[str]
    configured: bool
    worker_initialized: bool
    override_backends: dict[str, str]

    def __init__(self, app: Celery, **kwargs: Any) -> None: ...
    def import_default_modules(self) -> list[Any]: ...
    def autodiscover_tasks(
        self, packages: list[str], related_name: str = "tasks"
    ) -> None: ...
    def cmdline_config_parser(
        self,
        args: list[str],
        namespace: str = "celery",
        re_type: re.Pattern[str] = ...,
        extra_types: dict[str, Any] | None = None,
        override_types: dict[str, Any] | None = None,
    ) -> dict[str, Any]: ...
    @property
    def conf(self) -> Any: ...
    def config_from_object(self, obj: Any, silent: bool = False) -> None: ...
    @property
    def default_modules(self) -> set[str]: ...
    def find_module(self, module: str) -> ModuleType | None: ...
    def import_from_cwd(
        self, module: str, imp: Any | None = None, package: str | None = None
    ) -> ModuleType: ...
    def import_module(self, module: str, package: str | None = None) -> ModuleType: ...
    def import_task_module(self, module: str) -> ModuleType: ...
    def init_worker(self) -> None: ...
    def init_worker_process(self) -> None: ...
    def now(self, utc: bool = True) -> datetime: ...
    def on_process_cleanup(self) -> None: ...
    def on_task_init(self, task_id: str, task: Any) -> None: ...
    def on_worker_init(self) -> None: ...
    def on_worker_process_init(self) -> None: ...
    def on_worker_shutdown(self) -> None: ...
    def read_configuration(
        self, env: str = "CELERY_CONFIG_MODULE"
    ) -> dict[str, Any]: ...
    def shutdown_worker(self) -> None: ...



from vine.abstract import Thenable as Thenable
from vine.funtools import (
    ensure_promise as ensure_promise,
)
from vine.funtools import (
    maybe_promise as maybe_promise,
)
from vine.funtools import (
    ppartial as ppartial,
)
from vine.funtools import (
    preplace as preplace,
)
from vine.funtools import (
    starpromise as starpromise,
)
from vine.funtools import (
    transform as transform,
)
from vine.funtools import (
    wrap as wrap,
)
from vine.promises import promise as promise
from vine.synchronization import barrier as barrier

__all__ = [
    "Thenable",
    "barrier",
    "ensure_promise",
    "maybe_promise",
    "ppartial",
    "preplace",
    "promise",
    "starpromise",
    "transform",
    "wrap",
]



from collections.abc import Iterable, Mapping, Sequence
from datetime import datetime
from typing import (
    Any,
    Generic,
    Literal,
    TypeVar,
    overload,
)

__all__ = ("Context", "Task")

import billiard
import celery
import celery.result
import kombu
from celery import canvas
from celery.app.base import Celery
from celery.backends.base import Backend
from celery.canvas import Signature, xmap, xstarmap
from celery.exceptions import Retry
from celery.result import EagerResult
from celery.utils.threads import _LocalStack
from celery.worker.request import _DeliveryInfo
from typing_extensions import ParamSpec

_P = ParamSpec("_P")
_R_co = TypeVar("_R_co", covariant=True)
_SigR = TypeVar("_SigR")

class Context:
    args: Sequence[Any] | None
    callbacks: Sequence[Mapping[str, Any]] | None
    called_directly: bool
    chain: str | None
    chord: str | None
    correlation_id: str | None
    delivery_info: _DeliveryInfo | None
    errbacks: Sequence[Mapping[str, Any]] | None
    eta: int | None
    expires: int | None
    group: str | None
    group_index: int | None
    headers: dict[str, Any] | None
    hostname: str | None
    id: str | None
    ignore_result: bool
    is_eager: bool
    kwargs: Mapping[str, Any] | None
    logfile: str | None
    loglevel: int | None
    origin: Any
    parent_id: str | None
    properties: Any | None
    retries: int
    reply_to: Any
    replaced_task_nesting: int
    root_id: str | None
    shadow: Any
    taskset: str | None  # compat alias to group
    timelimit: tuple[int, int] | tuple[None, None] | None
    utc: bool | None
    stamped_headers: list[str] | None
    stamps: dict[str, Any] | None
    def __init__(self, *args: dict[str, Any], **kwargs: Any) -> None: ...
    def update(self, *args: dict[str, Any], **kwargs: Any) -> None: ...
    def clear(self) -> None: ...
    def get(self, key: str, default: Any = ...) -> Any: ...
    def as_execution_options(self) -> dict[str, Any]: ...
    @property
    def children(self) -> list[str]: ...

class Task(Generic[_P, _R_co]):
    # Class-level references
    MaxRetriesExceededError: type[Exception]
    OperationalError: type[Exception]
    Request: str
    Strategy: str

    # Flags
    __bound__: bool
    __trace__: Any
    __v2_compat__: bool

    # Task configuration
    name: str | None
    typing: bool
    max_retries: int | None
    default_retry_delay: int
    rate_limit: str | None
    ignore_result: bool
    trail: bool
    send_events: bool
    store_errors_even_if_ignored: bool
    serializer: str
    time_limit: int | None
    soft_time_limit: int | None
    autoregister: bool
    track_started: bool
    acks_late: bool | None
    acks_on_failure_or_timeout: bool | None
    reject_on_worker_lost: bool | None
    throws: tuple[type[Exception], ...]
    expires: float | datetime | None
    priority: int | None
    resultrepr_maxsize: int
    request_stack: _LocalStack[Context]
    abstract: bool
    store_eager_result: bool

    # Config mapping
    from_config: tuple[tuple[str, str], ...]

    def on_replace(self, sig: Signature[Any]) -> None: ...
    @classmethod
    def bind(cls, app: Celery) -> Celery: ...
    @classmethod
    def on_bound(cls, app: Celery) -> None: ...
    @property
    def app(self) -> Celery: ...
    @classmethod
    def annotate(cls) -> None: ...
    @classmethod
    def add_around(cls, attr: str, around: Any) -> None: ...
    # TODO(sbdchd): might be able to use overloads to handle the case where
    # `bind=True` passes in the first argument.
    def __call__(self, *args: _P.args, **kwargs: _P.kwargs) -> _R_co: ...
    def run(self, *args: _P.args, **kwargs: _P.kwargs) -> _R_co: ...
    def start_strategy(self, app: Celery, consumer: Any, **kwargs: Any) -> Any: ...
    def delay(
        self, *args: _P.args, **kwargs: _P.kwargs
    ) -> celery.result.AsyncResult[_R_co]: ...
    def apply_async(
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | list[Signature[Any]] | None = ...,
        link_error: Signature[Any] | list[Signature[Any]] | None = ...,
        shadow: str | None = ...,
        *,
        # options
        countdown: float = ...,
        eta: datetime = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
        ignore_result: bool = ...,
        time_limit: int = ...,
        soft_time_limit: int = ...,
    ) -> celery.result.AsyncResult[_R_co]: ...
    def shadow_name(
        self, args: tuple[Any, ...], kwargs: dict[str, Any], options: dict[str, Any]
    ) -> None: ...
    def signature_from_request(
        self,
        request: Context | None = ...,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        queue: str | None = ...,
        **extra_options: Any,
    ) -> Signature[_R_co]: ...
    subtask_from_request = signature_from_request
    def retry(
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        exc: Exception | None = ...,
        throw: bool = ...,
        eta: datetime | None = ...,
        countdown: float | None = ...,
        max_retries: int | None = ...,
        *,
        # options
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | list[Signature[Any]] | None = ...,
        link_error: Signature[Any] | list[Signature[Any]] | None = ...,
        shadow: str | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
        ignore_result: bool = ...,
        time_limit: int = ...,
        soft_time_limit: int = ...,
    ) -> Retry: ...
    def apply(
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        link: Signature[Any] | list[Signature[Any]] | None = ...,
        link_error: Signature[Any] | list[Signature[Any]] | None = ...,
        task_id: str | None = ...,
        retries: int | None = ...,
        throw: bool | None = ...,
        logfile: str | None = ...,
        loglevel: str | None = ...,
        headers: Mapping[str, str] | None = ...,
        *,
        # options
        ignore_result: bool = ...,
        exchange: str = ...,
        routing_key: str = ...,
        priority: int = ...,
    ) -> EagerResult[_R_co]: ...
    def AsyncResult(
        self, task_id: str, **kwargs: Any
    ) -> celery.result.AsyncResult[_R_co]: ...
    def signature(
        self, args: tuple[Any, ...] | None = ..., *starargs: Any, **starkwargs: Any
    ) -> Signature[_R_co]: ...
    def subtask(
        self, args: tuple[Any, ...] | None = ..., *starargs: Any, **starkwargs: Any
    ) -> Signature[_R_co]: ...
    def s(self, *args: Any, **kwargs: Any) -> Signature[_R_co]: ...
    def si(self, *args: _P.args, **kwargs: _P.kwargs) -> Signature[_R_co]: ...
    def chunks(self, it: Iterable[Any], n: int) -> canvas.chunks: ...
    def map(self, it: Iterable[Any]) -> xmap: ...
    def starmap(self, it: Iterable[Any]) -> xstarmap: ...
    def send_event(
        self,
        type_: str,
        retry: bool = ...,
        retry_policy: Mapping[str, int] | None = ...,
        **fields: Any,
    ) -> list[tuple[object, object]]: ...
    def replace(self, sig: Signature[_SigR]) -> _SigR: ...
    @overload
    def add_to_chord(
        self, sig: Signature[Any], lazy: Literal[True]
    ) -> celery.result.AsyncResult[Any]: ...
    @overload
    def add_to_chord(
        self, sig: Signature[Any], lazy: Literal[False] = ...
    ) -> EagerResult[Any]: ...
    def update_state(
        self,
        task_id: str | None = ...,
        state: str | None = ...,
        meta: dict[str, Any] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def before_start(
        self, task_id: str, args: tuple[Any, ...], kwargs: dict[str, Any]
    ) -> None: ...
    def on_success(
        self, retval: Any, task_id: str, args: tuple[Any, ...], kwargs: dict[str, Any]
    ) -> None: ...
    def on_retry(
        self,
        exc: Exception,
        task_id: str,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
        einfo: billiard.einfo.ExceptionInfo,
    ) -> None: ...
    def on_failure(
        self,
        exc: Exception,
        task_id: str,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
        einfo: billiard.einfo.ExceptionInfo,
    ) -> None: ...
    def after_return(
        self,
        status: str,
        retval: Any,
        task_id: str,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
        einfo: billiard.einfo.ExceptionInfo,
    ) -> None: ...
    def add_trail(self, result: Any) -> Any: ...
    def push_request(self, *args: _P.args, **kwargs: _P.kwargs) -> None: ...
    def pop_request(self) -> None: ...
    @property
    def request(self) -> Context: ...
    @property
    def backend(self) -> Backend: ...
    @backend.setter
    def backend(self, value: Backend) -> None: ...
    @property
    def __name__(self) -> str: ...



from collections.abc import Generator, Mapping, Sequence
from contextlib import contextmanager
from typing import Any

import pytest
from celery import Celery, Task
from celery.worker import WorkController

NO_WORKER: str | None

def pytest_configure(config: pytest.Config) -> None: ...
@contextmanager
def _create_app(
    enable_logging: bool = False,
    use_trap: bool = False,
    parameters: Mapping[str, Any] | None = None,
    **config: Any,
) -> Generator[Celery[Any], None, None]: ...
@pytest.fixture(scope="session")
def use_celery_app_trap() -> bool: ...
@pytest.fixture(scope="session")
def celery_session_app(
    request: Any,
    celery_config: Mapping[str, Any],
    celery_parameters: Mapping[str, Any],
    celery_enable_logging: bool,
    use_celery_app_trap: bool,
) -> Generator[Celery[Any]]: ...
@pytest.fixture(scope="session")
def celery_session_worker(
    request: pytest.FixtureRequest,
    celery_session_app: Celery[Any],
    celery_includes: Sequence[str],
    celery_class_tasks: str,
    celery_worker_pool: str,
    celery_worker_parameters: Mapping[str, Any],
) -> Generator[WorkController, None, None]: ...
@pytest.fixture(scope="session")
def celery_enable_logging() -> bool: ...
@pytest.fixture(scope="session")
def celery_includes() -> Sequence[str]: ...
@pytest.fixture(scope="session")
def celery_worker_pool() -> str | Any: ...
@pytest.fixture(scope="session")
def celery_config() -> Mapping[str, Any]: ...
@pytest.fixture(scope="session")
def celery_parameters() -> Mapping[str, Any]: ...
@pytest.fixture(scope="session")
def celery_worker_parameters() -> Mapping[str, Any]: ...
@pytest.fixture
def celery_app(
    request: pytest.FixtureRequest,
    celery_config: Mapping[str, Any],
    celery_parameters: Mapping[str, Any],
    celery_enable_logging: bool,
    use_celery_app_trap: bool,
) -> Generator[Celery[Any], None, None]: ...
@pytest.fixture(scope="session")
def celery_class_tasks() -> Sequence[type[Task[Any, Any]]]: ...
@pytest.fixture
def celery_worker(
    request: pytest.FixtureRequest,
    celery_app: Celery[Any],
    celery_includes: Sequence[str],
    celery_worker_pool: str,
    celery_worker_parameters: Mapping[str, Any],
) -> Generator[WorkController, None, None]: ...
@pytest.fixture
def depends_on_current_app(celery_app: Celery[Any]) -> None: ...



from collections.abc import Callable
from typing import TypeAlias

MatcherFunction: TypeAlias = Callable[[str, str], bool]

class MatcherNotInstalled(Exception): ...

class MatcherRegistry:
    MatcherNotInstalled: type[MatcherNotInstalled]
    matcher_pattern_first: list[str]
    _matchers: dict[str, MatcherFunction]
    _default_matcher: MatcherFunction | None

    def __init__(self) -> None: ...
    def register(self, name: str, matcher: MatcherFunction) -> None: ...
    def unregister(self, name: str) -> None: ...
    def _set_default_matcher(self, name: str) -> None: ...
    def match(
        self,
        data: bytes,
        pattern: bytes,
        matcher: str | None = ...,
        matcher_kwargs: dict[str, str] | None = ...,
    ) -> bool: ...

registry: MatcherRegistry

def match(
    data: bytes,
    pattern: bytes,
    matcher: str | None = ...,
    matcher_kwargs: dict[str, str] | None = ...,
) -> bool: ...
def register(name: str, matcher: MatcherFunction) -> None: ...
def unregister(name: str) -> None: ...
def register_glob() -> None: ...
def register_pcre() -> None: ...
